"""This module is used to gather enumerations related to the Change resource in Novu"""

from novu.enums.polyfill import StrEnum


class ChangeKind(StrEnum):
    """This enumeration define all kinds of change in Novu"""

    FEED = "Feed"
    """The change is related to a feed"""

    MESSAGE_TEMPLATE = "MessageTemplate"
    """The change is related to a message template"""

    LAYOUT = "Layout"
    """The change is related to a layout"""

    NOTIFICATION_TEMPLATE = "NotificationTemplate"
    """The change is related to a notification template"""

    NOTIFICATION_GROUP = "NotificationGroup"
    """The change is related to a notification group"""



"""This module is used to gather all DTO definitions related to the Change resource in Novu"""

import dataclasses
from typing import Any, List, Optional

from novu.dto.base import CamelCaseDto, DtoIterableDescriptor
from novu.enums import ChangeKind


@dataclasses.dataclass
class ChangeDetailDto(CamelCaseDto["ChangeDetailDto"]):
    """Definition of a change detail"""

    op: Optional[str] = None  # pylint: disable=C0103
    """The operation code of the change"""

    path: Optional[List[str]] = None
    """The path of the modification"""

    val: Optional[Any] = None
    """Value of the change, struct depend of the context"""


@dataclasses.dataclass
class ChangeDto(CamelCaseDto["ChangeDto"]):  # pylint: disable=R0902
    """Definition of a change"""

    _id: Optional[str] = None
    """Change ID in Novu internal storage system"""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    _entity_id: Optional[str] = None
    """Entity ID in Novu internal storage system"""

    _parent_id: Optional[str] = None
    """Parent ID in Novu internal storage system"""

    enabled: Optional[bool] = None
    """If the change was enabled."""

    type: Optional[ChangeKind] = None
    """Kind of change applied."""

    change: DtoIterableDescriptor[ChangeDetailDto] = DtoIterableDescriptor[ChangeDetailDto](
        default_factory=list, item_cls=ChangeDetailDto
    )
    """Details about a change"""

    created_at: Optional[str] = None
    """Date-time of the integration initial configuration"""

    updated_at: Optional[str] = None
    """Date-time of the last update of the integration"""


@dataclasses.dataclass
class PaginatedChangeDto(CamelCaseDto["PaginatedChangeDto"]):
    """Definition of paginated changes"""

    page: int = 0
    """Page number"""

    total_count: int = 0
    """Total count"""

    page_size: int = 0
    """Page size"""

    data: DtoIterableDescriptor[ChangeDto] = DtoIterableDescriptor[ChangeDto](default_factory=list, item_cls=ChangeDto)
    """Data"""



"""This module is used to gather all DTO definitions related to the Event resource in Novu"""

import dataclasses
from typing import Optional

from novu.dto.base import CamelCaseDto
from novu.enums.field import (
    FieldFilterPartOn,
    FieldFilterPartOperator,
    FieldFilterPartTimeOperator,
)


@dataclasses.dataclass
class FieldFilterPartDto(CamelCaseDto["FieldFilterPartDto"]):
    """Definition of a field filter part"""

    field: str
    """Field of the field filter part"""

    value: str
    """Value of the field filter part

    If the :attr:`on` is set to ``FieldFilterPartOn.IS_ONLINE_IN_LAST``, this value is a number.

    If the :attr:`on` is set to ``FieldFilterPartOn.IS_ONLINE``, this value is a boolean.
    """

    operator: FieldFilterPartOperator
    """Operator of the field filter part"""

    on: FieldFilterPartOn  # pylint: disable=C0103
    """On value of the field filter part"""

    webhook_url: Optional[str] = None
    """Webhook URL, only set if the :attr:`on` is set to ``FieldFilterPartOn.WEBHOOK``"""

    time_operator: Optional[FieldFilterPartTimeOperator] = None
    """Time operator, only set if the :attr:`on` is set to ``FieldFilterPartOn.IS_ONLINE_IN_LAST``"""



"""This module is used to gather all DTO definitions related to the Subscriber resource in Novu"""

import dataclasses
from typing import List, Optional

from novu.dto.base import CamelCaseDto, DtoDescriptor, DtoIterableDescriptor
from novu.enums.provider import ProviderIdEnum


@dataclasses.dataclass
class SubscriberPreferenceChannelDto(CamelCaseDto["SubscriberPreferenceChannelDto"]):
    """Definition of a channel activation state in subscriber's preference"""

    email: Optional[bool] = True
    """Email notification activated?"""

    sms: Optional[bool] = True
    """SMS notification activated?"""

    in_app: Optional[bool] = True  # FIXME: Find a way to not camelize this field
    """In APP notification activated?"""

    chat: Optional[bool] = True
    """Chat notification activated?"""

    push: Optional[bool] = True
    """Push notification activated?"""


@dataclasses.dataclass
class SubscriberPreferenceTemplateDto(CamelCaseDto["SubscriberPreferenceTemplateDto"]):
    """Definition of a template in subscriber's preference"""

    _id: Optional[str] = None
    """Subscriber preference template's ID"""

    name: Optional[str] = None
    """Name of the subscriber's preference template"""

    critical: Optional[bool] = None
    """Defines if the user's preferences will be ignored or not by Novu for the given template.

    By defining the template as critical, Novu considers that all steps should be executed ignoring user preferences.
    """


@dataclasses.dataclass
class SubscriberPreferencePreferenceDto(CamelCaseDto["SubscriberPreferencePreferenceDto"]):
    """Definition of subscriber's preference sub-struct"""

    enabled: bool
    """If the subscriber preference are enabled"""

    channels: DtoDescriptor[SubscriberPreferenceChannelDto] = DtoDescriptor[SubscriberPreferenceChannelDto](
        item_cls=SubscriberPreferenceChannelDto
    )
    """The activation states of the different channels"""


@dataclasses.dataclass
class SubscriberPreferenceDto(CamelCaseDto["SubscriberPreferenceDto"]):
    """Definition of subscriber's preference"""

    preference: DtoDescriptor[SubscriberPreferencePreferenceDto] = DtoDescriptor[SubscriberPreferencePreferenceDto](
        item_cls=SubscriberPreferencePreferenceDto
    )
    """Sub-struct in Novu for preference / global activation state"""

    template: DtoDescriptor[SubscriberPreferenceTemplateDto] = DtoDescriptor[SubscriberPreferenceTemplateDto](
        item_cls=SubscriberPreferenceTemplateDto
    )
    """The identifiers of the template linked to the preferences and its criticality"""


@dataclasses.dataclass
class SubscriberChannelSettingsCredentialsDto(CamelCaseDto["SubscriberChannelSettingsCredentialsDto"]):
    """Credentials payload for the specified provider."""

    webhook_url: Optional[str] = None
    """Webhook url used by chat app integrations. The webhook should be obtained from the chat app provider"""

    channel: Optional[str] = None
    """Channel specification for Mattermost chat notifications"""

    device_tokens: Optional[List[str]] = None
    """Contains an array of the subscriber device tokens for a given provider. Used on Push integrations"""


@dataclasses.dataclass
class SubscriberChannelSettingsDto(CamelCaseDto["SubscriberChannelSettingsDto"]):
    """Definition of channel settings for subscriber."""

    provider_id: ProviderIdEnum
    """The provider identifier for the credentials"""

    _integration_id: str
    """Id of the integration that is used for this channel"""

    credentials: DtoDescriptor[SubscriberChannelSettingsCredentialsDto] = DtoDescriptor[
        SubscriberChannelSettingsCredentialsDto
    ](item_cls=SubscriberChannelSettingsCredentialsDto)
    """Credentials of this channel"""

    integration_identifier: Optional[str] = None
    """The integration identifier"""


@dataclasses.dataclass
class SubscriberDto(CamelCaseDto["SubscriberDto"]):  # pylint: disable=R0902
    """Definition of subscriber"""

    camel_case_fields = [
        "subscriber_id",
        "email",
        "first_name",
        "last_name",
        "phone",
        "avatar",
        "locale",
        "channels",
        "data",
    ]
    # Actually, only these fields are editable in Novu, so prevent any activity on others

    subscriber_id: str
    """Subscriber's ID"""

    email: Optional[str] = None
    """Email of the subscriber"""

    first_name: Optional[str] = None
    """First name of the subscriber"""

    last_name: Optional[str] = None
    """Last name of the subscriber"""

    phone: Optional[str] = None
    """Phone number of the subscriber"""

    avatar: Optional[str] = None
    """profile picture (must be a public url to access the avatar)"""

    locale: Optional[str] = None
    """language code (we recommend the use of ISO 639 to define them)"""

    _id: Optional[str] = None
    """Subscriber ID in Novu internal storage system"""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    channels: DtoIterableDescriptor[SubscriberChannelSettingsDto] = DtoIterableDescriptor[SubscriberChannelSettingsDto](
        default_factory=list, item_cls=SubscriberChannelSettingsDto
    )
    """Subscriber provider specific credentials"""

    deleted: Optional[bool] = None
    """If the subscriber is deleted"""

    created_at: Optional[str] = None
    """Creation date of the subscriber"""

    updated_at: Optional[str] = None
    """Last update date of the subscriber"""

    is_online: Optional[bool] = None
    """If the subscriber is online"""

    last_online_at: Optional[str] = None
    """Last connection date of the subscriber"""

    data: Optional[dict] = None
    """Apart from the above fixed structured user data, this field contain
    any unstructured custom data such as user’s address, nationality, height, etc."""


@dataclasses.dataclass
class PaginatedSubscriberDto(CamelCaseDto["PaginatedSubscriberDto"]):
    """Definition of paginated subscribers"""

    page: int = 0
    """Page number"""

    total_count: int = 0
    """Total count"""

    page_size: int = 0
    """Page size"""

    data: DtoIterableDescriptor[SubscriberDto] = DtoIterableDescriptor[SubscriberDto](
        default_factory=list, item_cls=SubscriberDto
    )
    """Data"""


@dataclasses.dataclass
class BulkResultSubscriberDto(CamelCaseDto["BulkResultSubscriberDto"]):
    """Definition of paginated subscribers"""

    created: DtoIterableDescriptor[SubscriberDto] = DtoIterableDescriptor[SubscriberDto](
        default_factory=list, item_cls=SubscriberDto
    )
    """List of subscribers that were created during the operation."""

    updated: DtoIterableDescriptor[SubscriberDto] = DtoIterableDescriptor[SubscriberDto](
        default_factory=list, item_cls=SubscriberDto
    )
    """List of subscribers that were updated during the operation."""

    failed: DtoIterableDescriptor[SubscriberDto] = DtoIterableDescriptor[SubscriberDto](
        default_factory=list, item_cls=SubscriberDto
    )
    """List of subscribers whose creation (or update) failed."""



"""This module is used to defined all helpers to parse and send well-formatted data to the Novu API"""

import dataclasses
import re
from typing import ClassVar, Generic, Iterable, List, Optional, Type, TypeVar, Union

CAMELIZE_PATTERN = re.compile(r"(?:^|_)(.)")
UNDERSCORE_PATTERN_1 = re.compile(r"([A-Z]+)([A-Z][a-z])")
UNDERSCORE_PATTERN_2 = re.compile(r"([a-z\d])([A-Z])")


def snake_case_to_camel_case(string: str, upper_camel_case: bool = False) -> str:
    """Convert strings to CamelCase.

    Examples:
        >>> snake_case_to_camel_case("device_type")
        'DeviceType'
        >>> snake_case_to_camel_case("device_type", False)
        'deviceType'

    Args:
        string: The string to camelize
        upper_camel_case:
            If set to `True` :func:`snake_case_to_camel_case` converts strings to UpperCamelCase.
            If set to `False` :func:`snake_case_to_camel_case` produces lowerCamelCase.
            Defaults to `False`.
    """
    if upper_camel_case:
        return re.sub(CAMELIZE_PATTERN, lambda m: m.group(1).upper(), string)

    return string[0].lower() + snake_case_to_camel_case(string, True)[1:]


def camel_case_to_snake_case(word: str) -> str:
    """Make an underscored, lowercase form from the expression in the string.

    Example:
        >>> camel_case_to_snake_case("DeviceType")
        'device_type'
    """
    word = re.sub(UNDERSCORE_PATTERN_1, r"\1_\2", word)
    word = re.sub(UNDERSCORE_PATTERN_2, r"\1_\2", word)
    return word.replace("-", "_").lower()


_T = TypeVar("_T")


@dataclasses.dataclass
class CamelCaseDto(Generic[_T]):
    """A generic dataclass that allows to convert data from Novu API written in
    camel case to python (in snake_case) and back."""

    snake_case_to_camel_case_ignored: ClassVar[List[str]] = []
    """List of fields which we don't want to camelize."""

    camel_case_fields: ClassVar[Optional[List[str]]] = None
    """List of fields to define in the dict during :meth:`to_camel_case`.

    If this list is not defined, we have taken all the fields from the data class."""

    @classmethod
    def from_camel_case(cls: Type[_T], data: dict) -> _T:
        """Helper to parse a camel case dict"""
        fields = {f.name: f.type for f in dataclasses.fields(cls)}  # type: ignore[arg-type]
        kwargs = {}
        for key, val in data.items():
            _key = camel_case_to_snake_case(key)
            if _key in fields.keys():
                kwargs[_key] = fields[_key].from_camel_case(val) if hasattr(fields[_key], "from_camel_case") else val

        return cls(**kwargs)

    def to_camel_case(self) -> dict:
        """Helper to build a camel case dict"""
        if self.camel_case_fields:
            # pylint: disable=E1135
            return {
                snake_case_to_camel_case(k) if k not in self.snake_case_to_camel_case_ignored else k: v
                for k, v in dataclasses.asdict(self).items()
                if k in self.camel_case_fields
            }
        return {
            snake_case_to_camel_case(k) if k not in self.snake_case_to_camel_case_ignored else k: v
            for k, v in dataclasses.asdict(self).items()
        }


_C_co = TypeVar("_C_co", bound=CamelCaseDto, covariant=True)


class DtoDescriptor(Generic[_C_co]):
    """The Dto descriptor is used on :func:`dataclasses.dataclass` to help them parse sub-struct defined using
    the :class:`~novu.dto.base.CamelCaseDto` class during :meth:`~novu.dto.base.CamelCaseDto.from_camel_case` calls
    on Novu API response.

    Example:
        >>> @dataclasses.dataclass
        ... class SubscriberPreferenceDto(CamelCaseDto["SubscriberPreferenceDto"]):
        ...     template: DtoDescriptor[SubscriberPreferenceTemplateDto] = (
        ...         DtoDescriptor[SubscriberPreferenceTemplateDto](item_cls=SubscriberPreferenceTemplateDto)
        ...     )
    """

    def __init__(self, item_cls: Type[CamelCaseDto]):
        self._name: Optional[str] = None
        self._item_cls = item_cls

    def __set_name__(self, _, name: str):
        self._name = f"_{name}"

    def __get__(self, obj, _) -> Optional[_C_co]:
        if obj is None:
            return None

        return getattr(obj, self._name, None) if self._name else None

    def __set__(self, obj, value: Union[_C_co, dict]) -> None:
        if self._name:  # pragma: no branch (ignore because should never append as descriptor, branch is just for mypy)
            setattr(obj, self._name, self._item_cls.from_camel_case(value) if isinstance(value, dict) else value)


class DtoIterableDescriptor(Generic[_C_co]):
    """The Dto iterable descriptor is used on :func:`dataclasses.dataclass` to help them parse iterable sub-struct
    defined using the :class:`~novu.dto.base.CamelCaseDto` class during
    :meth:`~novu.dto.base.CamelCaseDto.from_camel_case` calls on Novu API response.

    Example:
        >>> @dataclasses.dataclass
        ... class PaginatedTopicDto(CamelCaseDto["PaginatedTopicDto"]):
        ...     data: DtoIterableDescriptor[TopicDto] = DtoIterableDescriptor[TopicDto](
        ...         default_factory=list, item_cls=TopicDto
        ...     )
    """

    def __init__(self, default_factory, item_cls: Type[_C_co]):
        self.default_factory = default_factory
        self._name: Optional[str] = None
        self._item_cls = item_cls

    def __set_name__(self, _, name):
        self._name = f"_{name}"

    def __get__(self, obj, _) -> Optional[Iterable[_C_co]]:
        if obj is None:
            return None

        return getattr(obj, self._name, None) if self._name else None

    def __set__(self, obj, value: Union[Iterable[_C_co], Iterable[dict]]) -> None:
        if self._name and value is not None:
            setattr(obj, self._name, [self._item_cls.from_camel_case(v) if isinstance(v, dict) else v for v in value])



"""This module is used to gather all DTO definitions related to the Execution Detail resource in Novu"""

import dataclasses
from typing import Optional

from novu.dto.base import CamelCaseDto
from novu.enums import ChannelExtended, ExecutionSource, ExecutionStatus


@dataclasses.dataclass
class ExecutionDetailDto(CamelCaseDto["ExecutionDetailDto"]):  # pylint: disable=R0902
    """Definition of an execution detail"""

    _id: Optional[str] = None
    """Execution detail ID in Novu internal storage system"""

    _job_id: Optional[str] = None
    """Job ID in Novu internal storage system"""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    _notification_id: Optional[str] = None
    """Notification ID in Novu internal storage system"""

    _notification_template_id: Optional[str] = None
    """Notification template ID in Novu internal storage system"""

    _subscriber_id: Optional[str] = None
    """Subscriber ID in Novu internal storage system"""

    _message_id: Optional[str] = None
    """Message ID in Novu internal storage system"""

    provider_id: Optional[str] = None
    """Provider ID in Novu internal storage system"""

    transaction_id: Optional[str] = None
    """Transaction ID in Novu internal storage system"""

    channel: Optional[ChannelExtended] = None
    """Channel used to the execution (or logical step like digest or trigger)"""

    detail: Optional[str] = None
    """Details of the execution"""

    source: Optional[ExecutionSource] = None
    """Source of the execution"""

    status: Optional[ExecutionStatus] = None
    """Status of the execution"""

    is_test: Optional[bool] = None
    """If this execution is related to a test"""

    is_retry: Optional[bool] = None
    """If this execution detail is related to a retry"""

    raw: Optional[str] = None
    """Raw details of the execution"""

    created_at: Optional[str] = None
    """Creation date of the execution detail"""

    updated_at: Optional[str] = None
    """Last update date of the execution detail"""



"""This module is used to gather all DTO definitions related to the Message resource in Novu"""

import dataclasses
from typing import List, Optional

from novu.dto.base import CamelCaseDto, DtoIterableDescriptor
from novu.enums import Channel, ProviderIdEnum


@dataclasses.dataclass
class MessageDto(CamelCaseDto["MessageDto"]):  # pylint: disable=R0902
    """Definition of an event"""

    identifier: Optional[str] = None
    """The message identifier"""

    _id: Optional[str] = None
    """Message Notification ID in Novu internal storage system"""

    _template_id: Optional[str] = None
    """Template ID in Novu internal storage system"""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    _message_template_id: Optional[str] = None
    """Message Template ID in Novu internal storage system"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    _subscriber_id: Optional[str] = None
    """Subscriber ID in Novu internal storage system"""

    _job_id: Optional[str] = None
    """Job ID in Novu internal storage system"""

    template_identifier: Optional[str] = None
    """Template ID in Novu internal storage system"""

    email: Optional[str] = None
    """Email of the subscriber triggered where the message has been sent"""

    subject: Optional[str] = None
    """Subject of the email sent"""

    cta: Optional[dict] = None
    """Definition of the call to action used on message"""

    channel: Optional[Channel] = None
    """Channel used for the message"""

    content: Optional[str] = None
    """Content of the message"""

    provider_id: Optional[ProviderIdEnum] = None
    """Provider ID used for the message"""

    device_tokens: Optional[List[dict]] = None
    """A list of device tokens used on the provider to send message"""

    seen: Optional[bool] = None
    """If the message has been seen."""

    read: Optional[bool] = None
    """If the message has been read"""

    status: Optional[str] = None
    """Status of the activity notification"""

    transaction_id: Optional[str] = None
    """Transaction ID in Novu internal storage system"""

    payload: Optional[dict] = None
    """Payload used during trigger"""

    created_at: Optional[str] = None
    """Creation date of the message"""

    updated_at: Optional[str] = None
    """Last update date of the message"""

    deleted: Optional[bool] = None
    """If the message is deleted"""

    last_read_date: Optional[str] = None
    """Timestamp of the last read event registered"""

    last_seen_date: Optional[str] = None
    """Timestamp of the last seen event registered"""


@dataclasses.dataclass
class PaginatedMessageDto(CamelCaseDto["PaginatedMessageDto"]):
    """Paginated message definition"""

    page: int = 0
    """Page number"""

    total_count: int = 0
    """Total count"""

    page_size: int = 0
    """Page size"""

    data: DtoIterableDescriptor[MessageDto] = DtoIterableDescriptor[MessageDto](
        default_factory=list, item_cls=MessageDto
    )
    """Data"""



"""
This module is used to define the ``EnvironmentApi``, a python wrapper to interact with ``Environments`` in Novu.
"""

from typing import Dict, Iterator, Optional

import requests

from novu.api.base import Api
from novu.constants import ENVIRONMENTS_ENDPOINT
from novu.dto import EnvironmentApiKeyDto, EnvironmentDto


class EnvironmentApi(Api):
    """This class aims to handle all API methods around environments in Novu"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._environment_url = f"{self._url}{ENVIRONMENTS_ENDPOINT}"

    def list(self) -> Iterator[EnvironmentDto]:
        """List existing environments

        Yields:
            Mapped environment
        """
        results = self.handle_request("GET", self._environment_url)["data"]
        for result in results:
            yield EnvironmentDto.from_camel_case(result)

    def create(self, name: str, parent_id: Optional[str] = None) -> EnvironmentDto:
        """Create a new environment using it's name

        Args:
            name: The name of the environment to create
            parent_id: The parent of the environment to create. Default to None.

        Returns:
            Mapped created environment
        """
        body: Dict[str, str] = {"name": name}
        if parent_id:
            body["parentId"] = parent_id

        return EnvironmentDto.from_camel_case(self.handle_request("POST", self._environment_url, body)["data"])

    def current(self) -> EnvironmentDto:
        """Retrieve the current environment

        Returns:
            Current environment mapped to DTO
        """
        return EnvironmentDto.from_camel_case(self.handle_request("GET", f"{self._environment_url}/me")["data"])

    def api_keys(self) -> Iterator[EnvironmentApiKeyDto]:
        """Remove a environment using it's identifier

        Yields:
            Mapped environment's api keys
        """
        results = self.handle_request("GET", f"{self._environment_url}/api-keys")["data"]
        for result in results:
            yield EnvironmentApiKeyDto.from_camel_case(result)

    def regenerate_api_key(self) -> Iterator[EnvironmentApiKeyDto]:
        """Regenerate an environment api key

        Yields:
            Mapped environment's api key
        """
        results = self.handle_request("POST", f"{self._environment_url}/api-keys/regenerate")["data"]
        for result in results:
            yield EnvironmentApiKeyDto.from_camel_case(result)



"""This module is used to gather all DTO definitions related to the Blueprint resource in Novu"""

import dataclasses
from typing import List, Optional

from novu.dto.base import CamelCaseDto, DtoIterableDescriptor


@dataclasses.dataclass
class BlueprintDto(CamelCaseDto["BlueprintDto"]):  # pylint: disable=R0902
    """Blueprint definition."""

    name: str
    """Blueprint name."""

    description: str
    """Blueprint description."""

    _id: Optional[str] = None
    """Blueprint ID in Novu internal storage system."""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system."""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system."""

    _notification_group_id: Optional[str] = None
    """Notification group ID in Novu internal storage system."""

    _creator_id: Optional[str] = None
    """Creator ID in Novu internal storage system."""

    created_at: Optional[str] = None
    """Creation date-time of the blueprint."""

    updated_at: Optional[str] = None
    """Last date-time when the blueprint was updated."""

    deleted: Optional[bool] = None
    """If the blueprint is deleted."""

    deleted_at: Optional[str] = None
    """Date-time of the removal of the blueprint."""

    deleted_by: Optional[str] = None
    """User who asked for the removal of the blueprint."""

    critical: Optional[bool] = None
    """If the blueprint is critical."""

    _parent_id: Optional[str] = None
    """Parent ID in Novu internal storage system."""

    active: Optional[bool] = None
    """Active status of the blueprint."""

    blueprint_id: Optional[str] = None
    """Blueprint ID in Novu internal storage system."""

    draft: Optional[bool] = None
    """Draft status of the blueprint."""

    is_blueprint: Optional[bool] = None
    """If the object is a blueprint."""

    notification_group: Optional[object] = None
    """Notification group object."""

    preference_settings: Optional[object] = None
    """Preference settings object."""

    steps: Optional[List[object]] = None
    """List of steps in the blueprint."""

    tags: Optional[List[str]] = None
    """List of tags associated with the blueprint."""

    triggers: Optional[List[object]] = None
    """List of triggers in the blueprint."""


@dataclasses.dataclass
class GroupedBlueprintDto(CamelCaseDto["GroupedBlueprintDto"]):
    """Grouped blueprint definition."""

    category: str
    """Blueprint category."""

    general: DtoIterableDescriptor[BlueprintDto] = DtoIterableDescriptor[BlueprintDto](
        default_factory=list, item_cls=BlueprintDto
    )
    """List of general blueprints in the category."""

    popular: DtoIterableDescriptor[BlueprintDto] = DtoIterableDescriptor[BlueprintDto](
        default_factory=list, item_cls=BlueprintDto
    )
    """List of popular blueprints in the category."""



"""This module is used to gather all Data Transfer Object definitions in the Novu SDK.

All definitions of format returned by the Novu API are here, which help us to instantiate and document them
for developer purpose (instead of getting raw dict without any hint about what is in it).
"""

from novu.dto.blueprint import BlueprintDto, GroupedBlueprintDto
from novu.dto.change import ChangeDetailDto, ChangeDto, PaginatedChangeDto
from novu.dto.environment import (
    EnvironmentApiKeyDto,
    EnvironmentDto,
    EnvironmentWidgetDto,
)
from novu.dto.event import EventDto
from novu.dto.execution_detail import ExecutionDetailDto
from novu.dto.feed import FeedDto
from novu.dto.field import FieldFilterPartDto
from novu.dto.integration import IntegrationChannelUsageDto, IntegrationDto
from novu.dto.layout import LayoutDto, LayoutVariableDto, PaginatedLayoutDto
from novu.dto.member import MemberDto, MemberInviteDto, MemberUserDto
from novu.dto.message import MessageDto, PaginatedMessageDto
from novu.dto.notification import (
    ActivityGraphStatesDto,
    ActivityNotificationDto,
    ActivityNotificationExecutionDetailResponseDto,
    ActivityNotificationJobResponseDto,
    ActivityNotificationStepResponseDto,
    ActivityNotificationSubscriberResponseDTO,
    ActivityNotificationTemplateResponseDto,
    ActivityNotificationTriggerResponseDto,
    PaginatedActivityNotificationDto,
)
from novu.dto.notification_group import (
    NotificationGroupDto,
    PaginatedNotificationGroupDto,
)
from novu.dto.notification_template import (
    NotificationStepDto,
    NotificationStepMetadataDto,
    NotificationTemplateDto,
    NotificationTemplateFormDto,
    NotificationTriggerDto,
    NotificationTriggerVariableDto,
    PaginatedNotificationTemplateDto,
)
from novu.dto.organization import (
    OrganizationBrandingDto,
    OrganizationDto,
    PartnerConfigurationDto,
)
from novu.dto.step_filter import StepFilterDto
from novu.dto.subscriber import (
    BulkResultSubscriberDto,
    PaginatedSubscriberDto,
    SubscriberChannelSettingsCredentialsDto,
    SubscriberChannelSettingsDto,
    SubscriberDto,
    SubscriberPreferenceChannelDto,
    SubscriberPreferenceDto,
    SubscriberPreferencePreferenceDto,
    SubscriberPreferenceTemplateDto,
)
from novu.dto.tenant import PaginatedTenantDto, TenantDto
from novu.dto.topic import PaginatedTopicDto, TopicDto, TriggerTopicDto

__all__ = [
    "ActivityGraphStatesDto",
    "ActivityNotificationDto",
    "ActivityNotificationExecutionDetailResponseDto",
    "ActivityNotificationJobResponseDto",
    "ActivityNotificationStepResponseDto",
    "ActivityNotificationSubscriberResponseDTO",
    "ActivityNotificationTemplateResponseDto",
    "ActivityNotificationTriggerResponseDto",
    "BlueprintDto",
    "BulkResultSubscriberDto",
    "ChangeDetailDto",
    "ChangeDto",
    "EnvironmentApiKeyDto",
    "EnvironmentDto",
    "EnvironmentWidgetDto",
    "EventDto",
    "ExecutionDetailDto",
    "FeedDto",
    "FieldFilterPartDto",
    "GroupedBlueprintDto",
    "IntegrationChannelUsageDto",
    "IntegrationDto",
    "LayoutDto",
    "LayoutVariableDto",
    "MemberDto",
    "MemberInviteDto",
    "MemberUserDto",
    "MessageDto",
    "NotificationGroupDto",
    "NotificationStepDto",
    "NotificationStepMetadataDto",
    "NotificationTemplateDto",
    "NotificationTemplateFormDto",
    "NotificationTriggerDto",
    "NotificationTriggerVariableDto",
    "OrganizationBrandingDto",
    "OrganizationDto",
    "PaginatedActivityNotificationDto",
    "PaginatedChangeDto",
    "PaginatedLayoutDto",
    "PaginatedMessageDto",
    "PaginatedNotificationGroupDto",
    "PaginatedNotificationTemplateDto",
    "PaginatedSubscriberDto",
    "PaginatedTenantDto",
    "PaginatedTopicDto",
    "PartnerConfigurationDto",
    "StepFilterDto",
    "SubscriberChannelSettingsCredentialsDto",
    "SubscriberChannelSettingsDto",
    "SubscriberDto",
    "SubscriberPreferenceChannelDto",
    "SubscriberPreferenceDto",
    "SubscriberPreferencePreferenceDto",
    "SubscriberPreferenceTemplateDto",
    "TenantDto",
    "TopicDto",
    "TriggerTopicDto",
]



"""
This module is used to define the ``SubscriberApi``, a python wrapper to interact with ``Subscribers`` in Novu.
"""

from typing import Dict, Iterator, List, Optional, Union

import requests

from novu.api.base import Api, PaginationIterator
from novu.constants import SUBSCRIBERS_ENDPOINT
from novu.dto.subscriber import (
    BulkResultSubscriberDto,
    PaginatedSubscriberDto,
    SubscriberDto,
    SubscriberPreferenceDto,
)
from novu.enums import Channel, ProviderIdEnum


class SubscriberApi(Api):
    """This class aims to handle all API methods around subscribers in API"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._subscriber_url = f"{self._url}{SUBSCRIBERS_ENDPOINT}"

    def list(self, page: Optional[int] = None) -> PaginatedSubscriberDto:
        """Method to list subscriber

        Args:
            page: The page number. Defaults to 0.

        Returns:
            Paginated subscriber
        """
        payload: Dict[str, int] = {}
        if page:
            payload["page"] = page

        return PaginatedSubscriberDto.from_camel_case(self.handle_request("GET", self._subscriber_url, payload=payload))

    def stream(self) -> PaginationIterator[SubscriberDto]:
        """Stream all existing subscribers into an iterator.

        Returns:
            An iterator on all subscribers available.
        """
        return PaginationIterator(self, SubscriberDto, self._subscriber_url)

    def create(self, subscriber: SubscriberDto) -> SubscriberDto:
        """Method to push a given subscriber instance to Novu

        Args:
            subscriber: The subscriber instance to push to Novu

        Returns:
            The instance retrieved from Novu (populated after creation)
        """
        return SubscriberDto.from_camel_case(
            self.handle_request("POST", self._subscriber_url, subscriber.to_camel_case()).get("data", {})
        )

    def bulk_create(self, subscribers: Iterator[SubscriberDto]) -> BulkResultSubscriberDto:
        """Using this endpoint you can create multiple subscribers at once, to avoid multiple calls to the API.

        The bulk API is limited to 500 subscribers per request.

        Args:
            subscribers: An iterator of subscribers instance to push to Novu

        Returns:
            Result of the bulk operation in Novu.
        """
        return BulkResultSubscriberDto.from_camel_case(
            self.handle_request(
                "POST",
                f"{self._subscriber_url}/bulk",
                {"subscribers": [subscriber.to_camel_case() for subscriber in subscribers]},
            ).get("data", {})
        )

    def get(self, subscriber_id: str) -> SubscriberDto:
        """Method to get a subscriber using his identifier

        Args:
            subscriber_id: The subscriber identifier

        Returns:
            The subscriber instance
        """
        return SubscriberDto.from_camel_case(
            self.handle_request("GET", f"{self._subscriber_url}/{subscriber_id}").get("data", {})
        )

    def put(self, subscriber: SubscriberDto) -> SubscriberDto:
        """Method to update a subscriber using his instance

        Args:
            subscriber: The subscriber instance to push

        Returns:
            Updated subscriber instance from Novu API
        """
        return SubscriberDto.from_camel_case(
            self.handle_request(
                "PUT", f"{self._subscriber_url}/{subscriber.subscriber_id}", subscriber.to_camel_case()
            ).get("data", {})
        )

    def delete(self, subscriber_id: str) -> None:
        """Method used to delete a subscriber using his identifier

        Args:
            subscriber_id: The subscriber identifier
        """
        self.handle_request("DELETE", f"{self._subscriber_url}/{subscriber_id}")

    def credentials(
        self,
        subscriber_id: str,
        provider_id: str,
        webhook_url: Optional[str] = None,
        device_tokens: Optional[List[str]] = None,
    ) -> SubscriberDto:
        """Update subscriber credentials associated to the delivery methods such as slack and push tokens

        Args:
            subscriber_id: The subscriber identifier
            provider_id: The provider name (e.g: slack)
            webhook_url: The webhook URL to set in the provider credentials. Defaults to None.
            device_tokens: A list of device tokens to set in the provider credentials. Defaults to None.

        Returns:
            Updated subscriber
        """
        credentials: Dict[str, Union[str, List[str]]] = {}
        payload = {"providerId": provider_id, "credentials": credentials}

        if webhook_url:
            credentials["webhookUrl"] = webhook_url
        if device_tokens:
            credentials["deviceTokens"] = device_tokens

        return SubscriberDto.from_camel_case(
            self.handle_request("PUT", f"{self._subscriber_url}/{subscriber_id}/credentials", payload).get("data", {})
        )

    def delete_credentials(
        self,
        subscriber_id: str,
        provider_id: ProviderIdEnum,
    ) -> None:
        """Delete subscriber credentials such as slack and expo tokens.

        Args:
            subscriber_id: The subscriber identifier
            provider_id: The provider identifier (e.g: slack)
        """

        self.handle_request("DELETE", f"{self._subscriber_url}/{subscriber_id}/credentials/{provider_id}")

    def online_status(self, subscriber_id: str, status: bool) -> SubscriberDto:
        """Used to update the subscriber is_online flag

        Args:
            subscriber_id: The subscriber identifier.
            status: The subscriber is_online flag value.

        Returns:
            Updated subscriber
        """
        return SubscriberDto.from_camel_case(
            self.handle_request(
                "PATCH", f"{self._subscriber_url}/{subscriber_id}/online-status", {"isOnline": status}
            ).get("data", {})
        )

    def preferences(self, subscriber_id: str) -> Iterator[SubscriberPreferenceDto]:
        """Get the subscriber preferences

        Args:
            subscriber_id: The subscriber identifier

        Yield:
            Iterator of subscriber preference
        """
        results = self.handle_request("GET", f"{self._subscriber_url}/{subscriber_id}/preferences").get("data", [])
        for result in results:
            yield SubscriberPreferenceDto.from_camel_case(result)

    def change_channel_preference(
        self, subscriber_id: str, template_id: str, channel: Channel, channel_enabled: bool
    ) -> SubscriberPreferenceDto:
        """Change the subscriber preference for a targeted channel of given template

        Args:
            subscriber_id: The subscriber on which we want to change preference
            template_id: The template on which we want to change preference
            channel: The channel on which we want to change preference
            channel_enabled: The new state of activation of the channel

        Returns:
            Updated subscriber preference of the targeted template
        """
        return SubscriberPreferenceDto.from_camel_case(
            self.handle_request(
                "PATCH",
                f"{self._subscriber_url}/{subscriber_id}/preferences/{template_id}",
                {"channel": {"type": channel, "enabled": channel_enabled}},
            ).get("data", {})
        )

    def change_preference_state(self, subscriber_id: str, template_id: str, state: bool) -> SubscriberPreferenceDto:
        """Change the subscriber preference state (enabled or disabled) for a given template

        Args:
            subscriber_id: The subscriber identifier
            template_id: The template identifier
            state: The state of the subscriber preference for given template

        Returns:
            Updated subscriber preference of the targeted template
        """
        return SubscriberPreferenceDto.from_camel_case(
            self.handle_request(
                "PATCH", f"{self._subscriber_url}/{subscriber_id}/preferences/{template_id}", {"enabled": state}
            ).get("data", {})
        )

    def unseen_notifications(self, subscriber_id: str) -> int:
        """Retrieve the number of unseen notification for subscribers feed

        Args:
            subscriber_id: The subscriber identifier

        Returns:
            The number of unseen notification
        """
        res = self.handle_request("GET", f"{self._subscriber_url}/{subscriber_id}/notifications/unseen")
        return res.get("data", {}).get("count", 0)



"""
This module is used to define the ``LayoutApi``, a python wrapper to interact with ``Layouts`` in Novu.
"""

from typing import Optional

import requests

from novu.api.base import Api
from novu.constants import LAYOUTS_ENDPOINT
from novu.dto.layout import LayoutDto, PaginatedLayoutDto


class LayoutApi(Api):
    """This class aims to handle all API methods around layout in API"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._layout_url = f"{self._url}{LAYOUTS_ENDPOINT}"

    def list(self, page: Optional[int] = None, limit: Optional[int] = None) -> PaginatedLayoutDto:
        """List existing layouts

        Args:
            page: Page in pagination. Defaults to None.
            limit: Size of the page in pagination. Defaults to None.

        Returns:
            Paginated list of layout
        """
        payload = {"page": page if page is not None else 0, "pageSize": limit if limit is not None else 100}

        return PaginatedLayoutDto.from_camel_case(self.handle_request("GET", self._layout_url, payload=payload))

    def create(self, layout: LayoutDto) -> LayoutDto:
        """Create a layout and return his identifier

        Args:
            layout: The instance of the layout to create

        Returns:
            The created layout identifier
        """
        return LayoutDto.from_camel_case(self.handle_request("POST", self._layout_url, layout.to_camel_case())["data"])

    def get(self, layout_id: str) -> LayoutDto:
        """Get a layout using ID

        Args:
            layout_id: The layout ID

        Returns:
            The layout instance
        """
        return LayoutDto.from_camel_case(self.handle_request("GET", f"{self._layout_url}/{layout_id}")["data"])

    def patch(self, layout: LayoutDto) -> LayoutDto:
        """Update a layout

        Args:
            layout: The instance of the layout to patch

        Returns:
            Updated layout instance
        """
        return LayoutDto.from_camel_case(
            self.handle_request("PATCH", f"{self._layout_url}/{layout._id}", layout.to_camel_case())["data"]
        )

    def delete(self, layout_id: str) -> None:
        """Remove a layout

        Args:
            layout_id: The ID of the layout to remove
        """
        self.handle_request("DELETE", f"{self._layout_url}/{layout_id}")

    def set_default(self, layout_id: str) -> None:
        """Set a layout as the default layout to use

        Args:
            layout_id: The layout ID
        """
        self.handle_request("POST", f"{self._layout_url}/{layout_id}/default")



"""This module is used to gather enumerations related to the Field resource in Novu"""

from novu.enums.polyfill import StrEnum


class FieldFilterPartOperator(StrEnum):
    """This enumeration define possible operator for a field filter part"""

    LARGER = "LARGER"
    """Larger operator (``>``)"""

    SMALLER = "SMALLER"
    """Smaller operator (``<``)"""

    LARGER_EQUAL = "LARGER_EQUAL"
    """Larger or equal operator (``>=``)"""

    SMALLER_EQUAL = "SMALLER_EQUAL"
    """Smaller or equal operator (``<=``)"""

    EQUAL = "EQUAL"
    """Equal operator (``==``)"""

    NOT_EQUAL = "NOT_EQUAL"
    """Not equal operator (``!=``)"""

    ALL_IN = "ALL_IN"
    """All in operator, which is like a python :func:`all`"""

    ANY_IN = "ANY_IN"
    """Any in operator, which is like a python :func:`any`"""

    NOT_IN = "NOT_IN"
    """Not equal operator (``not in``)"""

    BETWEEN = "BETWEEN"
    """Between operator (``lower_range <= item <= upper_range``)"""

    NOT_BETWEEN = "NOT_BETWEEN"
    """Not between operator (``not (lower_range <= item <= upper_range)``)"""

    LIKE = "LIKE"
    """LIKE operator (``sentence.find(item)``)"""

    NOT_LIKE = "NOT_LIKE"
    """Not LIKE operator (``not sentence.find(item)``)"""

    IN = "IN"
    """Membership operator (``in``)"""


class FieldFilterPartOn(StrEnum):
    """This enumeration define possible trigger (``on``) for a field filter part"""

    SUBSCRIBER = "subscriber"
    """Filter using the subscriber's data"""

    PAYLOAD = "payload"
    """Filter using the payload's data"""

    WEBHOOK = "webhook"
    """Filter using the webhook's data"""

    IS_ONLINE = "isOnline"
    """Allow to check if the subscriber is online"""

    IS_ONLINE_IN_LAST = "isOnlineInLast"
    """Allow to check if the subscriber is online in last X given minutes"""


class FieldFilterPartTimeOperator(StrEnum):
    """
    This enumeration define possible operator for a filed filter part which is used if
    ``FieldFilterPartDto.on`` is set to ``FieldFilterPartOn.IS_ONLINE_IN_LAST``"""

    MINUTES = "minutes"
    """The minutes operator"""

    HOURS = "hours"
    """The hours operator"""

    DAYS = "days"
    """The days operator"""



"""This module is used to gather all python wrappers used to describe resources in Novu API.

In this SDK, we choose to split the Novu API by business resource to simplify its complexity.
"""

from novu.api.blueprint import BlueprintApi
from novu.api.change import ChangeApi
from novu.api.environment import EnvironmentApi
from novu.api.event import EventApi
from novu.api.execution_detail import ExecutionDetailApi
from novu.api.feed import FeedApi
from novu.api.inbound_parse import InboundParseApi
from novu.api.integration import IntegrationApi
from novu.api.layout import LayoutApi
from novu.api.message import MessageApi
from novu.api.notification import NotificationApi
from novu.api.notification_group import NotificationGroupApi
from novu.api.notification_template import NotificationTemplateApi
from novu.api.organization import OrganizationApi
from novu.api.subscriber import SubscriberApi
from novu.api.tenant import TenantApi
from novu.api.topic import TopicApi

__all__ = [
    "BlueprintApi",
    "ChangeApi",
    "EnvironmentApi",
    "EventApi",
    "ExecutionDetailApi",
    "FeedApi",
    "InboundParseApi",
    "IntegrationApi",
    "LayoutApi",
    "MessageApi",
    "NotificationApi",
    "NotificationGroupApi",
    "NotificationTemplateApi",
    "OrganizationApi",
    "SubscriberApi",
    "TenantApi",
    "TopicApi",
]



"""This module is used to gather enumerations related to the Channel resource in Novu"""

from novu.enums.polyfill import StrEnum


class Channel(StrEnum):
    """This enumeration define all available channel in Novu"""

    EMAIL = "email"
    """Email channel (Custom SMTP, EmailJS, MailerSend, Mailjet, Outlook 365,
    Postmark, SendGrid, Sendinblue, Amazon SES, ...)"""

    SMS = "sms"
    """SMS channel (SMS77, AWS SNS, Telnyx, Twilio SMS, ...)"""

    IN_APP = "in_app"
    """In APP channel"""

    CHAT = "chat"
    """Chat channel (discord, MS Teams, Slack, ...)"""

    PUSH = "push"
    """Push channel (Expo, Firebase Cloud Messaging, ...)"""


class ChannelExtended(StrEnum):
    """
    This enumeration define all available channel in Novu and intermediate provider between channel.

    This definition is useful to determine if a block is a channel or a digest for example.
    """

    EMAIL = "email"
    """Email channel (Custom SMTP, EmailJS, MailerSend, Mailjet, Outlook 365,
    Postmark, SendGrid, Sendinblue, Amazon SES, ...)"""

    SMS = "sms"
    """SMS channel (SMS77, AWS SNS, Telnyx, Twilio SMS, ...)"""

    IN_APP = "in_app"
    """In APP channel"""

    CHAT = "chat"
    """Chat channel (discord, MS Teams, Slack, ...)"""

    PUSH = "push"
    """Push channel (Expo, Firebase Cloud Messaging, ...)"""

    DIGEST = "digest"
    """Used to aggregate message on certain conditions"""

    TRIGGER = "trigger"
    """Trigger to start the notification template"""

    DELAY = "delay"
    """A delay in the notification template"""



"""
This module is used to define the ``FeedApi``, a python wrapper to interact with ``Feeds`` in Novu.
"""

from typing import Iterator, Optional

import requests

from novu.api.base import Api
from novu.constants import FEEDS_ENDPOINT
from novu.dto.feed import FeedDto


class FeedApi(Api):
    """This class aims to handle all API methods around feeds in Novu"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._feed_url = f"{self._url}{FEEDS_ENDPOINT}"

    def list(self) -> Iterator[FeedDto]:
        """List existing feeds

        Yields:
            Mapped feed
        """
        results = self.handle_request("GET", self._feed_url)["data"]
        for result in results:
            yield FeedDto.from_camel_case(result)

    def create(self, name: str) -> FeedDto:
        """Create a new feed using it's name

        Args:
            name: The name of the feed to create

        Returns:
            Mapped created feed
        """
        return FeedDto.from_camel_case(self.handle_request("POST", self._feed_url, {"name": name})["data"])

    def delete(self, feed_id: str) -> None:
        """Remove a feed using it's identifier

        Args:
            feed_id: The feed identifier in Novu internal storage system (``_id``)
        """
        self.handle_request("DELETE", f"{self._feed_url}/{feed_id}")



"""
This module is used to define the ``InboundParseApi``, a python wrapper to interact with ``InboundParse`` in Novu.
"""

from typing import Optional

import requests

from novu.api.base import Api
from novu.constants import INBOUND_PARSE_ENDPOINT


class InboundParseApi(Api):
    """This class aims to handle all API methods around inbound-parse functionality in API"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._inbound_parse_url = f"{self._url}{INBOUND_PARSE_ENDPOINT}"

    def validate_mx_record_setup(self) -> bool:
        """Validate the mx record setup for the inbound parse functionality

        Returns:
            If the mx record setup is well setup or not.
        """
        return (
            self.handle_request("GET", f"{self._inbound_parse_url}/mx/status")
            .get("data", {})
            .get("mxRecordConfigured", False)
        )



"""This module is used to gather all DTO definitions related to the Notifications resource in Novu"""

import dataclasses
from typing import List, Optional

from novu.dto.base import CamelCaseDto, DtoDescriptor, DtoIterableDescriptor
from novu.enums import Channel


@dataclasses.dataclass
class ActivityNotificationSubscriberResponseDTO(CamelCaseDto["ActivityNotificationSubscriberResponseDTO"]):
    """Definition of the specific response for an activity notification subscriber request."""

    _id: str
    """The subscriber's unique ID"""

    email: Optional[str] = None
    """The subscriber's email"""

    first_name: Optional[str] = None
    """The subscriber's first name"""

    last_name: Optional[str] = None
    """The subscriber's last name"""

    phone: Optional[str] = None
    """The subscriber's phone number"""


@dataclasses.dataclass
class ActivityNotificationTriggerResponseDto(CamelCaseDto["ActivityNotificationTriggerResponseDto"]):
    """Definition of  the triggers for the notifications"""

    type: str
    """The type of trigger"""

    identifier: str
    """The identifier of the trigger"""

    variables: List[dict]
    """The variables for the trigger"""

    subscriber_variables: Optional[List[dict]] = None
    """The subscriber variables for the trigger"""


@dataclasses.dataclass
class ActivityNotificationTemplateResponseDto(CamelCaseDto["ActivityNotificationTemplateResponseDto"]):
    """Definition of  the template used to send the notification"""

    name: str
    """The name of the template"""

    triggers: DtoIterableDescriptor[ActivityNotificationTriggerResponseDto] = DtoIterableDescriptor[
        ActivityNotificationTriggerResponseDto
    ](default_factory=list, item_cls=ActivityNotificationTriggerResponseDto)
    """The triggers for the template"""

    _id: Optional[str] = None
    """The unique ID of the template"""


# pylint: disable=R0902
@dataclasses.dataclass
class ActivityNotificationExecutionDetailResponseDto(CamelCaseDto["ActivityNotificationExecutionDetailResponseDto"]):
    """Definition of  the execution of the notification"""

    _id: str
    """The unique ID of the execution"""

    _job_id: str
    """The ID of the job that executed the notification"""

    status: str
    """The status of the execution"""

    detail: str
    """The detail of the execution"""

    is_retry: bool
    """Whether the execution is a retry"""

    is_test: bool
    """Whether the execution is a test"""

    provider_id: dict
    """The ID of the provider that executed the notification"""

    source: str
    """The source of the execution"""

    raw: Optional[str] = None
    """The raw data of the execution"""


@dataclasses.dataclass
class ActivityNotificationStepResponseDto(CamelCaseDto["ActivityNotificationStepResponseDto"]):
    """Definition for the Notification for the Step Dto"""

    _id: str
    """The unique ID of the step"""

    active: bool
    """Whether the step is active"""

    filters: dict
    """The filters for the step"""

    template: Optional[dict] = None
    """The template for the step"""


@dataclasses.dataclass
class ActivityNotificationJobResponseDto(CamelCaseDto["ActivityNotificationJobResponseDto"]):  # pylint: disable=R0902
    """Definition of  the job that executed the notification"""

    _id: str
    """The unique ID of the job"""

    provider_id: dict
    """The ID of the provider that executed the job"""

    status: str
    """The status of the job"""

    step: DtoIterableDescriptor[ActivityNotificationStepResponseDto] = DtoIterableDescriptor[
        ActivityNotificationStepResponseDto
    ](default_factory=list, item_cls=ActivityNotificationStepResponseDto)
    """The step of the job"""

    type: DtoDescriptor[ActivityNotificationTriggerResponseDto] = DtoDescriptor[ActivityNotificationTriggerResponseDto](
        item_cls=ActivityNotificationTriggerResponseDto
    )
    execution_details: DtoIterableDescriptor[ActivityNotificationExecutionDetailResponseDto] = DtoIterableDescriptor[
        ActivityNotificationExecutionDetailResponseDto
    ](default_factory=list, item_cls=ActivityNotificationExecutionDetailResponseDto)
    """The execution details of the job"""

    digest: Optional[dict] = None
    """The digest of the job"""

    payload: Optional[dict] = None
    """The payload of the job"""


@dataclasses.dataclass
class ActivityNotificationDto(CamelCaseDto["ActivityNotificationDto"]):  # pylint: disable=R0902
    """Definition of  the notification"""

    _environment_id: str
    """The environment ID of the notification"""

    _organization_id: str
    """The organization ID of the notification"""

    transaction_id: str
    """The transaction ID of the notification"""

    created_at: Optional[str] = None
    """The creation date of the notification"""

    channels: Optional[str] = None
    """The channels of the notification"""

    subscriber: Optional[DtoDescriptor[ActivityNotificationSubscriberResponseDTO]] = DtoDescriptor[
        ActivityNotificationSubscriberResponseDTO
    ](item_cls=ActivityNotificationSubscriberResponseDTO)
    """The subscriber of the notification"""

    template: Optional[DtoDescriptor[ActivityNotificationTemplateResponseDto]] = DtoDescriptor[
        ActivityNotificationTemplateResponseDto
    ](item_cls=ActivityNotificationTemplateResponseDto)
    """The template of the notification"""

    jobs: Optional[DtoDescriptor[ActivityNotificationJobResponseDto]] = DtoDescriptor[
        ActivityNotificationJobResponseDto
    ](item_cls=ActivityNotificationJobResponseDto)
    """The jobs of the notification"""

    _id: Optional[str] = None
    """The unique ID of the notification"""


@dataclasses.dataclass
class ActivityGraphStatesDto(CamelCaseDto["ActivityGraphStatesDto"]):
    """Definition of an activity graph states in Novu."""

    _id: str
    """Activity graph states ID in Novu internal storage system"""

    count: int
    """Count."""

    templates: List[str]
    """Graph states' template."""

    channels: List[Channel]
    """Graph states' channel."""


@dataclasses.dataclass
class PaginatedActivityNotificationDto(CamelCaseDto["PaginatedActivityNotificationDto"]):
    """Definition of paginated subscribers"""

    page: int = 0
    """Page number"""

    has_more: bool = False
    """If their is more notifications available"""

    page_size: int = 0
    """Page size"""

    data: DtoIterableDescriptor[ActivityNotificationDto] = DtoIterableDescriptor[ActivityNotificationDto](
        default_factory=list, item_cls=ActivityNotificationDto
    )
    """Data"""



"""This module is used to gather enumerations related to the Event resource in Novu"""

from novu.enums.polyfill import StrEnum


class EventStatus(StrEnum):
    """This enumeration define possible status of an event"""

    PROCESSED = "processed"
    """The event have been processed"""

    TRIGGER_NOT_ACTIVE = "trigger_not_active"
    """The trigger you gave is not active"""

    TEMPLATE_NOT_FOUND = "template_not_found"
    """The template was not found"""

    SUBSCRIBER_ID_MISSING = "subscriber_id_missing"
    """The subscriber ID you gave was not found"""



"""
This module is used to define the ``OrganizationApi``, a python wrapper to interact with ``Organizations`` in Novu.
"""

from typing import Iterator, Optional

import requests

from novu.api.base import Api
from novu.constants import ORGANIZATION_ENDPOINT
from novu.dto.member import MemberDto
from novu.dto.organization import OrganizationBrandingDto, OrganizationDto


class OrganizationApi(Api):
    """This class aims to handle all API methods around organization in API"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._organization_url = f"{self._url}{ORGANIZATION_ENDPOINT}"

    def list(self) -> Iterator[OrganizationDto]:
        """Fetch all organizations

        Returns:
            Iterator on organizations
        """
        results = self.handle_request("GET", self._organization_url)["data"]
        for result in results:
            yield OrganizationDto.from_camel_case(result)

    def create(self, name: str, logo: Optional[str] = None) -> OrganizationDto:
        """Create a organization and return his instance

        Args:
            organization: The instance of the organization to create

        Returns:
            The created organization instance
        """
        payload = {"name": name}
        if logo:
            payload["logo"] = logo

        return OrganizationDto.from_camel_case(self.handle_request("POST", self._organization_url, payload)["data"])

    def current(self) -> OrganizationDto:
        """Fetch current organization details

        Returns:
            The current organization instance
        """
        return OrganizationDto.from_camel_case(self.handle_request("GET", f"{self._organization_url}/me")["data"])

    def rename(self, name: str) -> None:
        """Rename the organization

        Args:
            name: The new name of the currently selected organization
        """
        self.handle_request("PATCH", self._organization_url, {"name": name})

    def list_members(self) -> Iterator[MemberDto]:
        """Fetch all members of current organizations

        Returns:
            Iterator on members instances
        """
        results = self.handle_request("GET", f"{self._organization_url}/members")["data"]
        for result in results:
            yield MemberDto.from_camel_case(result)

    def remove_member(self, member_id: str) -> MemberDto:
        """Remove a member from the current organization

        Args:
            member_id: The ID of the member to remove

        Returns:
            Instance of the deleted member
        """
        return MemberDto.from_camel_case(
            self.handle_request("DELETE", f"{self._organization_url}/members/{member_id}")["data"]
        )

    def update_branding(self, branding: OrganizationBrandingDto) -> OrganizationBrandingDto:
        """Update branding information for the current organization

        Args:
            branding: Branding instance including all information required to update branding

        Returns:
            The instance of the updated branding information
        """
        return OrganizationBrandingDto.from_camel_case(
            self.handle_request("PUT", f"{self._organization_url}/branding", json=branding.to_camel_case())["data"]
        )



"""This module is used to gather all DTO definitions related to the Layout resource in Novu"""

import dataclasses
from typing import Optional

from novu.dto.base import CamelCaseDto, DtoIterableDescriptor
from novu.enums import Channel, TemplateVariableTypeEnum


@dataclasses.dataclass
class LayoutVariableDto(CamelCaseDto["LayoutVariableDto"]):
    """Layout variable definition"""

    name: str
    """Layout variable name"""

    type: TemplateVariableTypeEnum
    """Layout variable type"""

    _id: Optional[str] = None
    """Layout variable ID in the Novu storage system"""

    required: bool = False
    """If this variable is required in layout"""

    default_value: Optional[str] = None
    """The variable default value in layout"""


@dataclasses.dataclass
class LayoutDto(CamelCaseDto["LayoutDto"]):  # pylint: disable=R0902
    """Layout definition"""

    camel_case_fields = ["name", "description", "content", "variables", "is_default"]
    # Actually, only these fields are editable in Novu, so prevent any activity on others

    name: str
    """Layout name"""

    description: str
    """Layout description"""

    content: str
    """Layout content, must contains at least {{{body}}}}"""

    is_default: bool
    """Layout is the default layout in your environment"""

    variables: DtoIterableDescriptor[LayoutVariableDto] = DtoIterableDescriptor[LayoutVariableDto](
        default_factory=list, item_cls=LayoutVariableDto
    )
    """List of variables used in the layout"""

    _id: Optional[str] = None
    """Layout ID in Novu internal storage system"""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    _creator_id: Optional[str] = None
    """Creator ID in Novu internal storage system"""

    content_type: Optional[str] = None
    """Content type of the layout"""

    channel: Optional[Channel] = None
    """Channel of the layout"""

    created_at: Optional[str] = None
    """Creation date-time of the layout"""

    updated_at: Optional[str] = None
    """Last date-time when the layout was updated"""

    is_deleted: Optional[bool] = None
    """If the layout is deleted"""


@dataclasses.dataclass
class PaginatedLayoutDto(CamelCaseDto["PaginatedLayoutDto"]):
    """Paginated layout definition"""

    page: int = 0
    """Page number"""

    total_count: int = 0
    """Total count"""

    page_size: int = 0
    """Page size"""

    data: DtoIterableDescriptor[LayoutDto] = DtoIterableDescriptor[LayoutDto](default_factory=list, item_cls=LayoutDto)
    """Data"""



"""This module is used to gather all DTO definitions related to the Event resource in Novu"""

import dataclasses
from typing import List, Optional, Union

from novu.dto.base import CamelCaseDto
from novu.enums import EventStatus


@dataclasses.dataclass
class EventDto(CamelCaseDto["EventDto"]):
    """Definition of an event"""

    acknowledged: bool
    """If trigger was acknowledged or not."""

    status: EventStatus
    """Status for trigger."""

    transaction_id: str
    """Transaction id for trigger."""


@dataclasses.dataclass
class InputEventDto(CamelCaseDto["InputEventDto"]):
    """Definition of an event used as an input"""

    name: str
    """The name of the template trigger to activate."""

    recipients: Union[str, List[str]]
    """A subscriber ID (or a list of subscriber ID) to reach with this trigger."""

    payload: dict
    """A JSON serializable python dict to pass additional custom information."""

    overrides: Optional[dict] = None
    """A JSON serializable python dict used to override provider specific configurations."""

    transaction_id: Optional[str] = None
    """A unique ID for this transaction."""

    actor: Optional[str] = None
    """It is used to display the Avatar of the provided actor's subscriber id."""

    tenant: Optional[str] = None
    """It is used to specify a tenant context during trigger event.
    If a new tenant object is provided, we will create a new tenant.
    """



"""This module is used to define a configuration ``NovuConfig`` reusable through the package."""

from novu.helpers import Singleton


class NovuConfig(metaclass=Singleton):  # pylint: disable=R0903
    """Singleton used to configure globally URL and API_KEY for the Novu Client"""

    url: str
    """The URL of the Novu API to reach"""

    api_key: str
    """The API Key required to auth on the Novu API"""

    @classmethod
    def configure(cls, url: str, api_key: str):
        """Class method provided to initialise the singleton data.

        Args:
            url: The URL of the Novu API to reach.
            api_key: The API Key required to auth on the Novu API.
        """
        cls().url = url
        cls().api_key = api_key



"""This module is used to gather enumerations related to the Notification resource in Novu"""

from novu.enums.polyfill import StrEnum


class NotificationStepMetadataType(StrEnum):
    """This enumeration define possible type of step's metadata"""

    REGULAR = "regular"
    """Metadata for a digest in regular mode"""

    BACKOFF = "backoff"
    """Metadata for a digest in back-off mode"""

    SCHEDULED = "scheduled"
    """Metadata for a scheduled step"""


class NotificationStepMetadataUnit(StrEnum):
    """This enumeration define possible unit of step's metadata"""

    SECONDS = "seconds"
    """In seconds"""

    MINUTES = "minutes"
    """In minutes"""

    HOURS = "hours"
    """In hours"""

    DAYS = "days"
    """In days"""



"""This module is used to gather enumerations related to the Execution resource in Novu"""

from novu.enums.polyfill import StrEnum


class ExecutionSource(StrEnum):
    """This enumeration define all sources possible for an execution in Novu"""

    CREDENTIALS = "Credentials"
    """Execution detail source is credentials"""

    INTERNAL = "Internal"
    """Execution detail source is internal"""

    PAYLOAD = "Payload"
    """Execution detail source is payload"""

    WEBHOOK = "Webhook"
    """Execution detail source is webhook"""


class ExecutionStatus(StrEnum):
    """This enumeration define all status possible for an execution in Novu"""

    SUCCESS = "Success"
    """Execution is successful"""

    WARNING = "Warning"
    """Execution end with a warning"""

    FAILED = "Failed"
    """Execution failed"""

    PENDING = "Pending"
    """Execution is pending"""

    QUEUED = "Queued"
    """Execution is queued"""

    READ_CONFIRMATION = "ReadConfirmation"
    """Execution reading has been confirmed"""



"""
This module is used to define the ``NotificationTemplateApi``, a python wrapper
to interact with ``NotificationTemplate`` in Novu.
"""

from typing import Optional

import requests

from novu.api.base import Api, PaginationIterator
from novu.constants import NOTIFICATION_TEMPLATES_ENDPOINT
from novu.dto.notification_template import (
    NotificationTemplateDto,
    NotificationTemplateFormDto,
    PaginatedNotificationTemplateDto,
)


class NotificationTemplateApi(Api):
    """This class aims to handle all API methods around notification templates in API"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._notification_template_url = f"{self._url}{NOTIFICATION_TEMPLATES_ENDPOINT}"

    def list(self, page: Optional[int] = None, limit: Optional[int] = None) -> PaginatedNotificationTemplateDto:
        """Method to list notification templates

        Returns:
            Paginated notification templates
        """
        payload = {}
        if page is not None:
            payload["page"] = page
        if limit is not None:
            payload["limit"] = limit

        return PaginatedNotificationTemplateDto.from_camel_case(
            self.handle_request("GET", self._notification_template_url, payload=payload)
        )

    def stream(self) -> PaginationIterator[NotificationTemplateDto]:
        """Stream all existing workflows into an iterator.

        Returns:
            An iterator on all workflows available.
        """
        return PaginationIterator(self, NotificationTemplateDto, self._notification_template_url)

    def create(self, notification_template: NotificationTemplateFormDto) -> NotificationTemplateDto:
        """Create a template using the notification template form definition

        Args:
            notification_template: The notification template form definition

        Returns:
            Mapped notification template after it's creation
        """
        return NotificationTemplateDto.from_camel_case(
            self.handle_request("POST", self._notification_template_url, notification_template.to_camel_case())["data"]
        )

    def get(self, notification_template_id: str) -> NotificationTemplateDto:
        """Retrieve a notification template using it's Novu ID

        Args:
            notification_template_id: The Novu ID of the notification template

        Returns:
            Mapped notification template
        """
        return NotificationTemplateDto.from_camel_case(
            self.handle_request("GET", f"{self._notification_template_url}/{notification_template_id}")["data"]
        )

    def update(
        self, notification_template_id: str, notification_template: NotificationTemplateFormDto
    ) -> NotificationTemplateDto:
        """Full update of a notification template, using it's template form definition

        Args:
            notification_template_id: The Novu ID of the notification template
            notification_template: The notification template from instance

        Returns:
            Mapped notification template after it's update
        """
        return NotificationTemplateDto.from_camel_case(
            self.handle_request(
                "PUT",
                f"{self._notification_template_url}/{notification_template_id}",
                notification_template.to_camel_case(),
            )["data"]
        )

    def delete(self, notification_template_id: str) -> None:
        """Delete a notification template using it's ID

        Args:
            notification_template_id: The notification template ID
        """
        self.handle_request("DELETE", f"{self._notification_template_url}/{notification_template_id}")

    def update_status(self, notification_template_id: str, status: bool) -> NotificationTemplateDto:
        """Update a notification template status (active or not) using it's ID and the requested status

        Args:
            notification_template_id: The Novu ID of the notification template
            status: The activation status of the notification template

        Returns:
            Mapped notification template after it's update
        """
        return NotificationTemplateDto.from_camel_case(
            self.handle_request(
                "PUT", f"{self._notification_template_url}/{notification_template_id}/status", {"active": status}
            )["data"]
        )



"""This module is used to gather helpers reused through the package."""

import importlib
from typing import Dict


class Singleton(type):
    """Metaclass to use if you need a singleton on your class

    Example:
        >>> from novu.helpers import Singleton
        >>> class MySingleton(metaclass=Singleton)
        ...    pass
    """

    _instances: Dict[type, type] = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super().__call__(*args, **kwargs)
        return cls._instances[cls]


class SentryProxy(metaclass=Singleton):
    """Simple singleton to proxy all methods of Sentry SDK

    This class allows, in the application, to make sentry optional, in terms of installation, without effort.
    """

    def __init__(self) -> None:
        self.import_module()

    def import_module(self):
        """Method to try to load the sentry_sdk module or set it to None"""
        try:
            self.sentry_sdk = importlib.import_module("sentry_sdk")
        except ModuleNotFoundError:
            self.sentry_sdk = None

    def __getattr__(self, attr):
        if self.sentry_sdk:
            return getattr(self.sentry_sdk, attr, None)
        return lambda *args, **kwargs: None



"""This module is used to define the ``BlueprintApi``,
a python wrapper to interact with ``Blueprint`` in Novu."""

from typing import Optional

import requests

from novu.api.base import Api
from novu.constants import BLUEPRINTS_ENDPOINT
from novu.dto.blueprint import BlueprintDto, GroupedBlueprintDto


class BlueprintApi(Api):
    """This class aims to handle all API methods around blueprints in Novu"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._blueprint_url = f"{self._url}{BLUEPRINTS_ENDPOINT}"

    def get_by_id(self, template_id: str) -> BlueprintDto:
        """Get a blueprint by ID

        Args:
            template_id: The ID of the blueprint

        Returns:
            The blueprint instance
        """
        return BlueprintDto.from_camel_case(self.handle_request("GET", f"{self._blueprint_url}/{template_id}")["data"])

    def get_grouped_by_category(self) -> GroupedBlueprintDto:
        """Get blueprints grouped by category

        Returns:
            Grouped blueprints instance
        """
        return GroupedBlueprintDto.from_camel_case(
            self.handle_request("GET", f"{self._blueprint_url}/group-by-category")["data"]
        )



"""This module is used to defined an abstract class for all reusable methods to communicate with the Novu API"""

import copy
import logging
import os
from json.decoder import JSONDecodeError
from typing import Generic, List, Optional, Type, TypeVar, Union

import pkg_resources
import requests

from novu.config import NovuConfig
from novu.dto.base import CamelCaseDto
from novu.helpers import SentryProxy

LOGGER = logging.getLogger(__name__)

__version__ = pkg_resources.get_distribution("novu").version
_C_co = TypeVar("_C_co", bound=CamelCaseDto, covariant=True)


class PaginationIterator(Generic[_C_co]):  # pylint: disable=R0902
    """The class is a generic iterator which allow to iterate directly on result without
    looking for pagination during handling.

    Args:
        api: The api used to call
        item_class: The Dto to parse each items return by API.
        url: The URL to query during fetch.
        payload: Payload to query during fetch.
    """

    def __init__(self, api: "Api", item_class: Type[_C_co], url: str, payload: Optional[dict] = None):
        self.__item_class = item_class
        self.__api = api
        self.__url = url
        self.__payload = payload or {}

        self.__has_more = False
        self.__page = 0
        self.__limit = 10
        self.__index = 0
        self.__buffer: List[_C_co] = []

        self.__payload["limit"] = self.__limit
        self.__payload["page"] = self.__page
        self.__fetch_data()

    def __iter__(self) -> "PaginationIterator[_C_co]":
        return self

    def __next__(self) -> _C_co:
        return self.next()

    def next(self) -> _C_co:
        """Implementation of the next behavior for the iterator."""
        if self.__index < len(self.__buffer):
            result, self.__index = self.__buffer[self.__index], self.__index + 1
            return result

        if self.__has_more:
            self.__fetch_data()

            result, self.__index = self.__buffer[self.__index], self.__index + 1
            return result

        raise StopIteration()

    def __fetch_data(self):
        self.__payload["page"] = self.__page
        self.__page += 1

        data = self.__api.handle_request(
            method="GET",
            url=self.__url,
            payload=self.__payload,
        )

        self.__has_more = data.get("hasMore", data.get("totalCount", 0) > (self.__page + 1 * self.__limit))
        self.__buffer = list(map(self.__item_class.from_camel_case, data.get("data", [])))
        self.__index = 0


class Api:  # pylint: disable=R0903
    """Base class for all API in the Novu client"""

    requests_timeout: int = 5
    """This field allow you to change the :param:`~requests.request.timeout` params which is used during API calls."""

    session: Optional[requests.Session] = None
    """This field allow you to use a :class:`~requests.Session` during API calls."""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        config = NovuConfig()

        url = url or config.url
        api_key = api_key or config.api_key

        self._url = url
        self._headers = {
            "Authorization": f"ApiKey {api_key}",
            "User-Agent": f"novu/python@{__version__}",
        }

        self.requests_timeout = requests_timeout or int(os.getenv("NOVU_PYTHON_REQUESTS_TIMEOUT", "5"))
        self.session = session

    def handle_request(
        self,
        method: str,
        url: str,
        json: Optional[Union[dict, list]] = None,
        payload: Optional[dict] = None,
        headers: Optional[dict] = None,
        **kwargs,
    ) -> dict:
        """Handle a request to the API.

        This method can handle all cases of request and is used to authenticate the request and raise
        an error on bad status.

        Args:
            method: The HTTP method used during the request (e.g. "POST")
            url: The URL to reach during the request
            json: The body to send, in json format. Defaults to None.
            payload: Params to send, in json format. Defaults to None.
            headers: Headers to send, in json format. Defaults to None.

        Returns:
            Return parsed response.
        """
        if headers:
            _headers = copy.deepcopy(self._headers)
            _headers.update(copy.deepcopy(headers))
        else:
            _headers = self._headers

        res = (self.session.request if self.session else requests.request)(
            method=method,
            url=url,
            headers=_headers,
            json=json,
            params=payload,
            timeout=self.requests_timeout,
            **kwargs,
        )

        if not res.ok:
            try:
                detail = res.json()
                SentryProxy().set_extra("error_details", detail)
            except JSONDecodeError:
                pass
            res.raise_for_status()

        if res.status_code == 204:
            return {}

        return res.json()



"""This module is used to gather enumerations related to the Provider resource in Novu"""

from typing import Union

from novu.enums.polyfill import StrEnum


class CredentialsKeyEnum(StrEnum):
    """This enumeration define possible key in a Novu credentials dict"""

    API_KEY = "apiKey"
    """Api key"""

    USER = "user"
    """User"""

    SECRET_KEY = "secretKey"  # nosec: B105
    """Secret key"""

    DOMAIN = "domain"
    """Domain"""

    PASSWORD = "password"  # nosec: B105
    """Password"""

    HOST = "host"
    """Host"""

    PORT = "port"
    """Port"""

    SECURE = "secure"
    """Secure"""

    REGION = "region"
    """Region"""

    ACCOUNT_SID = "accountSid"
    """Account SID"""

    MESSAGE_PROFILE_ID = "messageProfileId"
    """Message profile ID"""

    TOKEN = "token"  # nosec: B105
    """Token"""

    FROM = "from"
    """From"""

    SENDER_NAME = "senderName"
    """Sender name"""

    APPLICATION_ID = "applicationId"
    """Application ID"""

    CLIENT_ID = "clientId"
    """Client ID"""

    PROJECT_NAME = "projectName"
    """Project name"""

    SERVICE_ACCOUNT = "serviceAccount"
    """Service account"""

    BASE_URL = "baseUrl"
    """Base URL"""

    WEBHOOK_URL = "webhookUrl"
    """Webhook URL"""


class EmailProviderIdEnum(StrEnum):
    """This enumeration define possible email provider ID.

    For more details about their configuration, see https://docs.novu.co/channels/email/
    """

    EMAILJS = "emailjs"
    """EmailJS mail provider, see """

    MAILGUN = "mailgun"
    """MailGun mail provider"""

    MAILJET = "mailjet"
    """MailJet mail provider"""

    MANDRILL = "mandrill"
    """Mandrill mail provider"""

    CUSTOM_SMTP = "nodemailer"
    """Custom SMTP provider, for example a nodemailer"""

    POSTMARK = "postmark"
    """Postmark mail provider"""

    SENDGRID = "sendgrid"
    """SendGrid mail provider"""

    SENDINBLUE = "sendinblue"
    """SendInBlue mail provider"""

    SES = "ses"
    """Amazon SES mail provider"""

    NETCORE = "netcore"
    """NetCore mail provider"""

    INFOBIP = "infobip-email"
    """InfoBIP mail provider"""

    MAILERSEND = "mailersend"
    """Mailer Send provider"""

    CLICKATELL = "clickatell"
    """Click-a-tell mail provider"""

    OUTLOOK365 = "outlook365"
    """Outlook 365 mail provider"""

    NOVU = "novu-email"
    """Novu internal mail provider"""


class SmsProviderIdEnum(StrEnum):
    """This enumeration define possible sms provider ID"""

    NEXMO = "nexmo"
    """Nexmo sms provider"""

    PLIVO = "plivo"
    """Plivo sms provider"""

    SMS77 = "sms77"
    """SMS 77 provider"""

    SNS = "sns"
    """Amazon SNS provider"""

    TELNYX = "telnyx"
    """Telnyx sms provider"""

    TWILIO = "twilio"
    """Twilio sms provider"""

    GUPSHUP = "gupshup"
    """Gupshup sms provider"""

    FIRETEXT = "firetext"
    """Firetext sms provider"""

    INFOBIP = "infobip-sms"
    """InfoBIP sms provider"""

    BURST_SMS = "burst-sms"
    """BurstSMS provider"""

    CLICKATELL = "clickatell"
    """Click-a-tell sms provider"""


class ChatProviderIdEnum(StrEnum):
    """This enumeration define possible chat provider ID"""

    SLACK = "slack"
    """Slack provider for chat messages"""

    DISCORD = "discord"
    """Discord provider for chat messages"""

    MS_TEAMS = "msteams"
    """MS Teams provider for chat messages"""


class PushProviderIdEnum(StrEnum):
    """This enumeration define possible push provider ID"""

    FCM = "fcm"
    """FCM notification push provider"""

    APNS = "apns"
    """APNS notification push provider"""

    EXPO = "expo"
    """EXPO notification push provider"""


class InAppProviderIdEnum(StrEnum):
    """This enumeration define possible in_app provider ID"""

    NOVU = "novu"
    """The only in-app provider, Novu"""


ProviderIdEnum = Union[
    SmsProviderIdEnum, PushProviderIdEnum, ChatProviderIdEnum, EmailProviderIdEnum, InAppProviderIdEnum
]
"""Type to define the notion of provider ID, which is one of sms, push, chat, email, or in_app provider ID."""



"""This module is used to gather enumerations related to the Provider resource in Novu"""

from novu.enums.polyfill import StrEnum


class OrganizationBrandingDirection(StrEnum):
    """This enumeration define possible direction in a Novu organization branding"""

    LEFT_TO_RIGHT = "ltr"
    """Left to right"""

    RIGHT_TO_LEFT = "trl"
    """Right to left"""


class PartnerTypeEnum(StrEnum):
    """This enumeration define possible partner type"""

    VERCEL = "vercel"
    """Vercel partner type"""



"""This module is used to gather all DTO definitions related to the Notification Template resource in Novu"""

import dataclasses
from typing import List, Optional

from novu.dto.base import CamelCaseDto, DtoDescriptor, DtoIterableDescriptor
from novu.dto.notification_group import NotificationGroupDto
from novu.dto.step_filter import StepFilterDto
from novu.dto.subscriber import SubscriberPreferenceChannelDto
from novu.enums.notification import (
    NotificationStepMetadataType,
    NotificationStepMetadataUnit,
)


@dataclasses.dataclass
class NotificationStepMetadataDto(CamelCaseDto["NotificationStepMetadataDto"]):
    """Definition of metadata for a notification step"""

    type: NotificationStepMetadataType
    """Define the kind of metadata (regular, backoff or scheduled)"""

    amount: Optional[int] = None
    """The amount of the notification step metadata"""

    unit: Optional[NotificationStepMetadataUnit] = None
    """The unit of the notification step metadata"""

    digest_key: Optional[str] = None
    """If specified, the digest engine will group the events based on the digestKey and subscriberId,
    otherwise the digest engine will group the events based only on the subscriberId.

    The digest key might come useful when you want a particular subscriber to get events grouped on a
    custom field. For example when an actor likes the user's post, you might want to digest based on the post_id key.
    """

    delay_path: Optional[str] = None
    """Payload path will determine the path in payload for the scheduled date.

    That path must be included in your payload on trigger execution and must be a date in strict ISO format.
    """

    backoff_unit: Optional[NotificationStepMetadataUnit] = None
    """The backoff unit of the notification step metadata"""

    update_mode: Optional[bool] = None
    """The update mode of the notification step metadata"""


@dataclasses.dataclass
class NotificationStepDto(CamelCaseDto["NotificationStepDto"]):  # pylint: disable=R0902
    """Definition of a notification step"""

    _id: Optional[str] = None
    """Notification Step ID in Novu internal storage system"""

    _template_id: Optional[str] = None
    """Notification Template ID in Novu internal storage system"""

    _parent_id: Optional[str] = None
    """Notification Step ID in Novu internal storage system (parent of this step)"""

    active: Optional[bool] = None
    """If this step is active"""

    should_stop_on_fail: Optional[bool] = None
    """The flow should stop the flow if this step fail ?"""

    template: Optional[dict] = None
    """Message template of the step"""

    filters: DtoIterableDescriptor[StepFilterDto] = DtoIterableDescriptor[StepFilterDto](
        default_factory=list, item_cls=StepFilterDto
    )
    """Filters of this template"""

    metadata: Optional[NotificationStepMetadataDto] = None
    """Metadata of this notification step"""

    replay_callback: Optional[dict] = None
    """Definition of the replay callback"""


@dataclasses.dataclass
class NotificationTriggerVariableDto(CamelCaseDto["NotificationTriggerVariableDto"]):
    """Definition of a trigger's variable for notification template"""

    name: str
    """Notification trigger variable's name"""


@dataclasses.dataclass
class NotificationTriggerDto(CamelCaseDto["NotificationTriggerDto"]):
    """Definition of a trigger for a notification template"""

    type: str
    """Notification trigger's type"""

    identifier: str
    """Notification trigger's identifier"""

    variables: DtoIterableDescriptor[NotificationTriggerVariableDto] = DtoIterableDescriptor[
        NotificationTriggerVariableDto
    ](default_factory=list, item_cls=NotificationTriggerVariableDto)
    """Variables of a notification trigger"""

    subscriber_variables: DtoIterableDescriptor[NotificationTriggerVariableDto] = DtoIterableDescriptor[
        NotificationTriggerVariableDto
    ](default_factory=list, item_cls=NotificationTriggerVariableDto)
    """Subscriber's variables of a notification trigger"""


@dataclasses.dataclass
class NotificationTemplateFormDto(CamelCaseDto["NotificationTemplateDto"]):  # pylint: disable=R0902
    """Definition of a notification template form (create / update)"""

    name: str
    """Name of the template"""

    notification_group_id: str
    """Notification Group ID from Novu internal storage system"""

    description: Optional[str] = None
    """A description of the template"""

    tags: Optional[List[str]] = None
    """Notification template's tags"""

    steps: DtoIterableDescriptor[NotificationStepDto] = DtoIterableDescriptor[NotificationStepDto](
        default_factory=list, item_cls=NotificationStepDto
    )
    """Steps of the template, which describe a step of your notification flow"""

    active: Optional[bool] = None
    """If the template is active"""

    critical: Optional[bool] = None
    """If the template should bypass user preferences"""

    draft: Optional[bool] = None
    """If the template is a draft"""

    preference_settings: Optional[DtoDescriptor[SubscriberPreferenceChannelDto]] = DtoDescriptor[
        SubscriberPreferenceChannelDto
    ](item_cls=SubscriberPreferenceChannelDto)
    """Preference settings of the notification template"""


@dataclasses.dataclass
class NotificationTemplateDto(CamelCaseDto["NotificationTemplateDto"]):  # pylint: disable=R0902
    """Definition of a notification template"""

    name: str
    """Name of the template"""

    description: Optional[str] = None
    """A description of the template"""

    tags: Optional[List[str]] = None
    """Notification template's tags"""

    steps: DtoIterableDescriptor[NotificationStepDto] = DtoIterableDescriptor[NotificationStepDto](
        default_factory=list, item_cls=NotificationStepDto
    )
    """Steps of the template, which describe a step of your notification flow"""

    active: Optional[bool] = None
    """If the template is active"""

    critical: Optional[bool] = None
    """If the template should bypass user preferences"""

    draft: Optional[bool] = None
    """If the template is a draft"""

    preference_settings: Optional[DtoDescriptor[SubscriberPreferenceChannelDto]] = DtoDescriptor[
        SubscriberPreferenceChannelDto
    ](item_cls=SubscriberPreferenceChannelDto)
    """Preference settings of the notification template"""

    triggers: DtoIterableDescriptor[NotificationTriggerDto] = DtoIterableDescriptor[NotificationTriggerDto](
        default_factory=list, item_cls=NotificationTriggerDto
    )
    """Usable triggers to launch this notification template"""

    _id: Optional[str] = None
    """Notification Template ID in Novu internal storage system"""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    _notification_group_id: Optional[str] = None
    """Notification Group ID in Novu internal storage system"""

    _parent_id: Optional[str] = None
    """Parent ID in Novu internal storage system"""

    _creator_id: Optional[str] = None
    """Creator ID in Novu internal storage system"""

    notification_group: Optional[DtoDescriptor[NotificationGroupDto]] = DtoDescriptor[NotificationGroupDto](
        item_cls=NotificationGroupDto
    )
    """Notification group linked to the notification template"""

    created_at: Optional[str] = None
    """The notification template's creation date"""

    updated_at: Optional[str] = None
    """The last notification template's update date"""

    deleted_at: Optional[str] = None
    """The notification template has been deleted at"""

    deleted: Optional[bool] = None
    """If the notification template is deleted"""


@dataclasses.dataclass
class PaginatedNotificationTemplateDto(CamelCaseDto["PaginatedNotificationTemplateDto"]):
    """Definition of paginated subscribers"""

    page: int = 0
    """Page number"""

    total_count: int = 0
    """Total count"""

    page_size: int = 0
    """Page size"""

    data: DtoIterableDescriptor[NotificationTemplateDto] = DtoIterableDescriptor[NotificationTemplateDto](
        default_factory=list, item_cls=NotificationTemplateDto
    )
    """Data"""



"""This module is used to gather all DTO definitions related to the Member resource in Novu."""

import dataclasses
from typing import List, Optional

from novu.dto.base import CamelCaseDto, DtoDescriptor
from novu.enums import MemberRole, MemberStatus


@dataclasses.dataclass
class MemberUserDto(CamelCaseDto["MemberUserDto"]):
    """Member user definition"""

    first_name: str
    """Member first name"""

    last_name: str
    """Member last name"""

    email: str
    """Member email"""

    _id: Optional[str] = None
    """User ID in Novu internal storage system"""


@dataclasses.dataclass
class MemberInviteDto(CamelCaseDto["MemberInviteDto"]):
    """Member invite definition"""

    email: str
    """Invited email"""

    token: str
    """Invite token"""

    invitation_date: str
    """Date-time of the invitation"""

    answer_date: Optional[str] = None
    """Date-time of the answer to the invitation"""

    _invite_id: Optional[str] = None
    """Invitation ID in Novu internal storage system"""


@dataclasses.dataclass
class MemberDto(CamelCaseDto["MemberDto"]):
    """Member definition"""

    _id: Optional[str] = None
    """Member ID in Novu internal storage system"""

    _user_id: Optional[str] = None
    """User ID in Novu internal storage system"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    user: Optional[DtoDescriptor[MemberUserDto]] = DtoDescriptor[MemberUserDto](item_cls=MemberUserDto)
    roles: Optional[List[MemberRole]] = None
    invite: Optional[DtoDescriptor[MemberInviteDto]] = DtoDescriptor[MemberInviteDto](item_cls=MemberInviteDto)
    member_status: Optional[MemberStatus] = None



"""This module is used to gather all DTO definitions related to the Integration resource in Novu"""

import dataclasses
from typing import Dict, Optional, Union

from novu.dto.base import CamelCaseDto
from novu.enums import Channel, CredentialsKeyEnum, ProviderIdEnum


@dataclasses.dataclass
class IntegrationDto(CamelCaseDto["IntegrationDto"]):  # pylint: disable=R0902
    """Definition of an integration"""

    camel_case_fields = ["provider_id", "channel", "credentials", "active"]
    # Actually, only these fields are editable in Novu, so prevent any activity on others

    provider_id: ProviderIdEnum
    """Provider ID, which is one of predefined available enum"""

    channel: Channel
    """Integration channel"""

    active: bool
    """If the provider is active"""

    credentials: Optional[Dict[CredentialsKeyEnum, Union[str, bool]]] = None
    """Credentials of the provider"""

    _id: Optional[str] = None
    """Integration ID in Novu internal storage system"""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    created_at: Optional[str] = None
    """Date-time of the integration initial configuration"""

    updated_at: Optional[str] = None
    """Date-time of the last update of the integration"""

    deleted_at: Optional[str] = None
    """Date-time of the removal of the integration"""

    deleted_by: Optional[str] = None
    """User how ask for the removal of the integration"""

    deleted: Optional[bool] = None
    """If the integration is deleted"""


@dataclasses.dataclass
class IntegrationChannelUsageDto(CamelCaseDto["IntegrationChannelUsageDto"]):
    """Definition of an integration usage (scoped to a channel)"""

    count: Optional[int]
    """Usage count of the channel"""

    limit: Optional[int]
    """Usage limit of the channel"""



"""
This module is used to define the ``IntregrationApi``, a python wrapper to interact with ``Integrations`` in Novu.
"""

from typing import Iterator, Optional

import requests

from novu.api.base import Api
from novu.constants import INTEGRATIONS_ENDPOINT
from novu.dto.integration import IntegrationChannelUsageDto, IntegrationDto
from novu.enums import Channel, ProviderIdEnum


class IntegrationApi(Api):
    """This class aims to handle all API methods around integrations in API"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._integration_url = f"{self._url}{INTEGRATIONS_ENDPOINT}"

    def list(self, only_active: bool = False) -> Iterator[IntegrationDto]:
        """List configured integrations

        Args:
            only_active: Allow to retrieve only active integrations. Defaults to False.

        Yields:
            Instance of integrations
        """
        url = self._integration_url
        if only_active:
            url += "/active"

        results = self.handle_request("GET", url)["data"]
        for result in results:
            yield IntegrationDto.from_camel_case(result)

    def create(self, integration: IntegrationDto, check: bool = True) -> IntegrationDto:
        """Create a provider integration configuration

        Args:
            integration: Integration instance that you want to create.
            check:
                If you want the Novu server to check his connection using given integration configuration.
                Defaults to True.

        Returns:
            The instance of the created integration
        """
        payload = integration.to_camel_case()
        payload["check"] = check if check is not None else True

        return IntegrationDto.from_camel_case(self.handle_request("POST", self._integration_url, payload)["data"])

    def status(self, provider_id: ProviderIdEnum) -> bool:
        """Get webhook support status for a given provider

        Args:
            provider_id: The provider ID

        Returns:
            If the provider support webhook
        """
        return self.handle_request("GET", f"{self._integration_url}/webhooks/provider/{provider_id}/status")["data"]

    def update(self, integration: IntegrationDto, check: bool = True) -> IntegrationDto:
        """Update an integration configuration

        Args:
            integration: The instance of the integration to update
            check:
                If you want the Novu server to check his connection using given integration configuration.
                Defaults to True.

        Returns:
            The instance of the updated integration
        """
        payload = integration.to_camel_case()
        payload["check"] = check if check is not None else True

        return IntegrationDto.from_camel_case(
            self.handle_request("PUT", f"{self._integration_url}/{integration._id}", payload)["data"]
        )

    def delete(self, integration_id: str) -> None:
        """Delete an integration definition

        Args:
            integration_id: The integration ID
        """
        self.handle_request("DELETE", f"{self._integration_url}/{integration_id}")

    def limit(self, channel: Channel) -> IntegrationChannelUsageDto:
        """Get limit of the given channel (and usage)

        Args:
            channel: The channel to retrieve limits

        Returns:
            Channel usage definition (including usage and limit)
        """
        return IntegrationChannelUsageDto.from_camel_case(
            self.handle_request("GET", f"{self._integration_url}/{channel}/limit")["data"]
        )

    def set_primary(self, integration_id: str) -> IntegrationDto:
        """Set an integration as primary
        Args:
            integration_id: The integration ID
        Raises:
            HTTPError: Raise a 400 error when provided integration identifier is not found in the application
            HTTPError: Raise a 401 error when you are not authorized to do this action
        Returns:
            The instance of the updated integration
        """
        return IntegrationDto.from_camel_case(
            self.handle_request("POST", f"{self._integration_url}/{integration_id}/set-primary")["data"]
        )



"""
Novu SDK
========

Provides
  1. Wrapper to interact with Novu API for each resource
  2. DTO dataclasses to parse Novu resources.
  3. Enumerations from Novu.

Available subpackages
---------------------

api
    Wrapper to interact with Novu API for each resource
dto
    DTO dataclasses to parse Novu resources.
enums
    Enumerations from Novu.
"""

from novu import api, dto, enums
from novu.config import NovuConfig

__all__ = ["api", "dto", "enums", "NovuConfig"]



"""This module is used to gather all DTO definitions related to the Feed resource in Novu"""

import dataclasses
from typing import Optional

from novu.dto.base import CamelCaseDto


@dataclasses.dataclass
class FeedDto(CamelCaseDto["FeedDto"]):  # pylint: disable=R0902
    """Definition of a feed"""

    name: str
    """Name of the feed"""

    identifier: Optional[str] = None
    """Feed identifier, should be the given name set by default in Novu"""

    _id: Optional[str] = None
    """Feed ID in Novu internal storage system"""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    deleted: Optional[bool] = None
    """If the field is deleted"""

    created_at: Optional[str] = None
    """Creation date of the field"""

    updated_at: Optional[str] = None
    """Last update date of the feed"""

    id: Optional[str] = None  # pylint: disable=C0103
    """Feed ID in Novu internal storage system"""



"""
This module is used to define the ``ChangeApi``, a python wrapper to interact with ``Changes`` in Novu.
"""

from typing import Dict, Generator, List, Optional, Union

import requests

from novu.api.base import Api
from novu.constants import CHANGES_ENDPOINT
from novu.dto.change import ChangeDto, PaginatedChangeDto


class ChangeApi(Api):
    """This class aims to handle all API methods around changes in API"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._change_url = f"{self._url}{CHANGES_ENDPOINT}"

    def list(
        self, page: Optional[int] = None, limit: Optional[int] = None, promoted: str = "false"
    ) -> PaginatedChangeDto:
        """List existing changes

        Args:
            page: Page to retrieve. Defaults to None.
            limit: Size of the page to retrieve. Defaults to None.
            promoted: Required string to retrieve changes.

        Returns:
            Paginated list of change
        """
        payload: Dict[str, Union[int, str]] = {"promoted": promoted}
        if page:
            payload["page"] = page
        if limit:
            payload["limit"] = limit

        return PaginatedChangeDto.from_camel_case(self.handle_request("GET", self._change_url, payload=payload))

    def count(self) -> int:
        """Get the number of changes

        Returns:
            Number of changes
        """
        return self.handle_request("GET", f"{self._change_url}/count")["data"]

    def apply(self, change_id: str) -> PaginatedChangeDto:
        """Apply one change using its ID

        Args:
            change_ids: A change ID to apply.

        Returns:
            Paginated changes
        """
        return PaginatedChangeDto.from_camel_case(self.handle_request("POST", f"{self._change_url}/{change_id}/apply"))

    def bulk_apply(self, change_ids: List[str]) -> Generator[ChangeDto, None, None]:
        """Apply a list of changes using their IDs

        Args:
            change_ids: The list of IDs to apply

        Yields:
            Parsed Change from results list
        """
        results = self.handle_request("POST", f"{self._change_url}/bulk/apply", {"changeIds": change_ids})["data"]
        for result in results:
            for sub_result in result:
                yield ChangeDto.from_camel_case(sub_result)



"""This module is used to gather all enumerations defined by Novu in Python format to be reused by developers."""

from novu.enums.change import ChangeKind
from novu.enums.channel import Channel, ChannelExtended
from novu.enums.event import EventStatus
from novu.enums.execution import ExecutionSource, ExecutionStatus
from novu.enums.field import (
    FieldFilterPartOn,
    FieldFilterPartOperator,
    FieldFilterPartTimeOperator,
)
from novu.enums.member import MemberRole, MemberStatus
from novu.enums.notification import (
    NotificationStepMetadataType,
    NotificationStepMetadataUnit,
)
from novu.enums.organization import OrganizationBrandingDirection, PartnerTypeEnum
from novu.enums.provider import (
    ChatProviderIdEnum,
    CredentialsKeyEnum,
    EmailProviderIdEnum,
    InAppProviderIdEnum,
    ProviderIdEnum,
    PushProviderIdEnum,
    SmsProviderIdEnum,
)
from novu.enums.step_filter import StepFilterType, StepFilterValue
from novu.enums.template import TemplateVariableTypeEnum

__all__ = [
    "ChangeKind",
    "Channel",
    "ChannelExtended",
    "ChatProviderIdEnum",
    "CredentialsKeyEnum",
    "EmailProviderIdEnum",
    "EventStatus",
    "ExecutionSource",
    "ExecutionStatus",
    "FieldFilterPartOn",
    "FieldFilterPartOperator",
    "FieldFilterPartTimeOperator",
    "InAppProviderIdEnum",
    "MemberRole",
    "MemberStatus",
    "NotificationStepMetadataType",
    "NotificationStepMetadataUnit",
    "OrganizationBrandingDirection",
    "PartnerTypeEnum",
    "ProviderIdEnum",
    "PushProviderIdEnum",
    "SmsProviderIdEnum",
    "StepFilterType",
    "StepFilterValue",
    "TemplateVariableTypeEnum",
]



"""
This module is used to define the ``TopicApi``, a python wrapper to interact with ``Topics`` in Novu.
"""

from typing import Dict, List, Optional, Tuple, Union

import requests

from novu.api.base import Api
from novu.constants import TOPICS_ENDPOINT
from novu.dto.topic import PaginatedTopicDto, TopicDto


class TopicApi(Api):
    """This class aims to handle all API methods around topics in API"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._topic_url = f"{self._url}{TOPICS_ENDPOINT}"

    def list(
        self, page: Optional[int] = None, limit: Optional[int] = None, key: Optional[str] = None
    ) -> PaginatedTopicDto:
        """List existing topics

        Args:
            page: Page to retrieve. Defaults to None.
            limit: Size of the page to retrieve. Defaults to None.
            key: Filter list by a topic key. Defaults to None.

        Returns:
            Paginated list of topic
        """
        payload: Dict[str, Union[int, str]] = {}
        if page:
            payload["page"] = page
        if limit:
            payload["limit"] = limit
        if key:
            payload["key"] = key

        return PaginatedTopicDto.from_camel_case(self.handle_request("GET", self._topic_url, payload=payload))

    def create(self, key: str, name: str) -> TopicDto:
        """Create a topic

        Args:
            key: The topic key to create
            name: The topic name to create

        Returns:
            Created topic (without name ?)
        """
        return TopicDto.from_camel_case(
            self.handle_request("POST", self._topic_url, TopicDto(key, name).to_camel_case())["data"]
        )

    def get(self, key: str) -> TopicDto:
        """Retrieve a Topic using his key

        .. note:: This method is the easiest way to get the list of subscribers.

        Args:
            key: The topic key (ID in Novu)

        Returns:
            Topic
        """
        return TopicDto.from_camel_case(self.handle_request("GET", f"{self._topic_url}/{key}")["data"])

    # FIXME: In documentation, doesn't return anything. But in real life ?
    def subscribe(self, key: str, subscribers: Union[List[str], str]) -> Tuple[List[str], Dict[str, List[str]]]:
        """Subscribe a list of subscribers to a topic

        Args:
            key: The key of the topic to subscribe
            subscribers: The list of subscribers to subscribe

        Returns:
            First element returned is a list of succeeded subscriptions.

            Second element returned is a dict of failed subscriptions
            (key the reason and value contains a list of reference which fail for the reason).
        """
        payload = {"subscribers": subscribers if isinstance(subscribers, list) else [subscribers]}
        result = self.handle_request("POST", f"{self._topic_url}/{key}/subscribers", payload)
        return result["data"].get("succeeded", []), result["data"].get("failed", {})

    def unsubscribe(self, key: str, subscribers: Union[List[str], str]) -> None:
        """Unsubscribe a list of subscribers form a topic

        Args:
            key: The key of the topic to unsubscribe
            subscribers: The list of subscribers to unsubscribe
        """
        payload = {"subscribers": subscribers if isinstance(subscribers, list) else [subscribers]}
        self.handle_request("POST", f"{self._topic_url}/{key}/subscribers/removal", payload)

    def rename(self, key: str, name: str) -> TopicDto:
        """Rename a topic

        Args:
            key: The key of the topic to rename
            name: The new name of the topic

        Returns:
            Renamed topic definition
        """
        return TopicDto.from_camel_case(
            self.handle_request("PATCH", f"{self._topic_url}/{key}", {"name": name})["data"]
        )

    def delete(self, key: str) -> None:
        """Delete a topic by its topic key if it has no subscribers

        Args:
            Key: The topic key to delete
        """

        self.handle_request("DELETE", f"{self._topic_url}/{key}")

    def subscribed(self, key: str, subscriber_id: str) -> bool:
        """Allows you to check if a subscriber has subscribed to the provided topic.

        Args:
            key: The topic's key
            subscriber_id: The subscriber identifier

        Returns:
            bool: Result of the check
        """
        try:
            self.handle_request("GET", f"{self._topic_url}/{key}/subscribers/{subscriber_id}")
            return True
        except requests.exceptions.HTTPError as exc:
            if exc.response.status_code == 404:
                return False
            raise exc



"""This module is used to gather enumerations related to the Step Filter resource in Novu"""

from novu.enums.polyfill import StrEnum


class StepFilterType(StrEnum):
    """This enumeration define possible type for a step filter"""

    BOOL = "BOOLEAN"
    """Boolean filter"""

    TEXT = "TEXT"
    """Text filter"""

    DATE = "DATE"
    """Date filter"""

    NUMBER = "NUMBER"
    """Number filter"""

    STATEMENT = "STATEMENT"
    """Statement filter"""

    LIST = "LIST"
    """List filter"""

    MULTI_LIST = "MULTI_LIST"
    """Multi-list filter"""

    GROUP = "GROUP"
    """Group filter"""


class StepFilterValue(StrEnum):
    """This enumeration define possible value for a step filter"""

    AND = "AND"
    """And operator"""

    OR = "OR"
    """Or operator"""



"""
This module is used to define the ``ExecutionDetailApi``, a python wrapper
to interact with ``ExecutionDetails`` in Novu.
"""

from typing import Iterator, Optional

import requests

from novu.api.base import Api
from novu.constants import EXECUTION_DETAILS_ENDPOINT
from novu.dto import ExecutionDetailDto


class ExecutionDetailApi(Api):
    """This class aims to handle all API methods around execution details in Novu"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._execution_detail_url = f"{self._url}{EXECUTION_DETAILS_ENDPOINT}"

    def list(self, notification_id: str, subscriber_id: str) -> Iterator[ExecutionDetailDto]:
        """List existing execution details using provided notification and subscriber IDs

        Args:
            notification_id: The notification ID in Novu
            subscriber_id: The subscriber ID in Novu

        Yields:
            Mapped execution detail
        """
        results = self.handle_request(
            "GET",
            self._execution_detail_url,
            payload={"notificationId": notification_id, "subscriberId": subscriber_id},
        )["data"]
        for result in results:
            yield ExecutionDetailDto.from_camel_case(result)



"""
This module is used to define the ``MessageApi``, a python wrapper to interact with ``Messages`` in Novu.
"""

from typing import Dict, Optional, Union

import requests

from novu.api.base import Api, PaginationIterator
from novu.constants import MESSAGES_ENDPOINT
from novu.dto.message import MessageDto, PaginatedMessageDto


class MessageApi(Api):
    """This class aims to handle all API methods around messages in Novu"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._message_url = f"{self._url}{MESSAGES_ENDPOINT}"

    def list(
        self,
        limit: int = 10,
        page: int = 0,
        channel: Optional[str] = None,
        subscriber_id: Optional[str] = None,
        transaction_id: Optional[str] = None,
    ) -> PaginatedMessageDto:
        """List messages

        Args:
            limit: The number of messages to fetch, defaults to 10
            page: The page to fetch, defaults to 0
            channel: The channel for the messages you wish to list. Defaults to None.
            subscriber_id: The subscriberId for the subscriber you like to list messages for
            transaction_id: The transactionId for the messages you wish to list. Defaults to None.

        Returns:
            Returned a paginated struct containing retrieved messages
        """
        payload: Dict[str, Union[str, int]] = {"limit": limit, "page": page}

        if channel:
            payload["channel"] = channel
        if subscriber_id:
            payload["subscriberId"] = subscriber_id
        if transaction_id:
            payload["transactionId"] = transaction_id

        return PaginatedMessageDto.from_camel_case(self.handle_request("GET", self._message_url, payload=payload))

    def stream(
        self, channel: Optional[str] = None, subscriber_id: Optional[str] = None
    ) -> PaginationIterator[MessageDto]:
        """Stream all existing messages into an iterator.

        Args:
            channel: The channel for the messages you wish to list. Defaults to None.
            subscriber_id: The subscriberId for the subscriber you like to list messages for

        Returns:
            An iterator on all messages available.
        """
        payload = {}

        if channel:
            payload["channel"] = channel
        if subscriber_id:
            payload["subscriberId"] = subscriber_id

        return PaginationIterator(self, MessageDto, self._message_url, payload=payload)

    def delete(self, message_id: str) -> bool:
        """Deletes a message entity from the Novu platform

        Args:
            message_id: The message ID to delete

        Returns:
            This function answer if the delete is a success by parsing the acknowledged field in response.
        """
        return self.handle_request("DELETE", f"{self._message_url}/{message_id}")["data"].get("acknowledged", False)



"""This module is used to gather all DTO definitions related to the Topic resource in Novu"""

import dataclasses
from typing import List, Optional

from novu.dto.base import CamelCaseDto, DtoIterableDescriptor


@dataclasses.dataclass
class TriggerTopicDto(CamelCaseDto["TriggerTopicDto"]):
    """Topic definition for trigger"""

    topic_key: str
    """Topic key"""

    type: str
    """Topic type"""


@dataclasses.dataclass
class TopicDto(CamelCaseDto["TopicDto"]):
    """Topic definition"""

    camel_case_fields = ["key", "name"]
    # Actually, only these fields are editable in Novu, so prevent any activity on others

    key: str
    """Topic key"""

    name: Optional[str] = None
    """Name, required during creation"""

    _id: Optional[str] = None
    """Topic ID in Novu internal system"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    subscribers: Optional[List[str]] = None
    """List of subscribers in the topic."""


@dataclasses.dataclass
class PaginatedTopicDto(CamelCaseDto["PaginatedTopicDto"]):
    """Paginated topic definition"""

    page: int = 0
    """Page number"""

    total_count: int = 0
    """Total count"""

    page_size: int = 0
    """Page size"""

    data: DtoIterableDescriptor[TopicDto] = DtoIterableDescriptor[TopicDto](default_factory=list, item_cls=TopicDto)
    """Data"""



"""This module is used to gather all DTO definitions related to the Notification Group resource in Novu"""

import dataclasses
from typing import Optional

from novu.dto.base import CamelCaseDto, DtoIterableDescriptor


@dataclasses.dataclass
class NotificationGroupDto(CamelCaseDto["NotificationGroupDto"]):
    """Definition of a notification group"""

    name: str
    """Notification group's name"""

    _id: Optional[str] = None
    """Notification Group ID in Novu internal storage system"""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    _parent_id: Optional[str] = None
    """Parent ID in Novu internal storage system"""

    created_at: Optional[str] = None
    """Creation date of the notification group"""

    updated_at: Optional[str] = None
    """Last update date of the notification group"""


@dataclasses.dataclass
class PaginatedNotificationGroupDto(CamelCaseDto["PaginatedNotificationGroupDto"]):
    """Definition of paginated notification groups"""

    page: int = 0
    """Page number"""

    total_count: int = 0
    """Total count"""

    page_size: int = 0
    """Page size"""

    data: DtoIterableDescriptor[NotificationGroupDto] = DtoIterableDescriptor[NotificationGroupDto](
        default_factory=list, item_cls=NotificationGroupDto
    )
    """Data"""



"""This module is used to define the ``NotificationAPI`, a python wrapper
to interact with ``Notifications`` in Novu.
"""

from typing import Iterator, List, Optional, Tuple

import requests

from novu.api.base import Api, PaginationIterator
from novu.constants import NOTIFICATION_ENDPOINT
from novu.dto.notification import (
    ActivityGraphStatesDto,
    ActivityNotificationDto,
    PaginatedActivityNotificationDto,
)


class NotificationApi(Api):
    """This class aims to handle all API methods around notifications in API"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._notification_url = f"{self._url}{NOTIFICATION_ENDPOINT}"

    def list(
        self,
        channels: Optional[List[str]] = None,
        templates: Optional[List[str]] = None,
        emails: Optional[List[str]] = None,
        subscriber_ids: Optional[List[str]] = None,
        search: Optional[str] = None,
        page: Optional[int] = 0,
        transaction_id: Optional[str] = None,
    ) -> PaginatedActivityNotificationDto:
        """Trigger an event to get all notifications.

        Args:
            channels: A required parameter, should be an array of strings representing
                      available notification channels, such as "in_app", "email", "sms",
                      "chat", and "push".

            templates: A required parameter, should be an array of strings representing
                       the notification templates.

            emails: A required parameter, should be an array of strings representing
                    the email addresses associated with the notification.

            subscriber_ids: An array of subscriber's identifier associated with searched
                            notifications.

            search: A required parameter, should be a string representing the search query.

            page: An optional parameter with a default value of 0, representing the page
                  number for search results.

            transaction_id: A required parameter, should be a string representing the
                            transaction ID associated with the notification.

        Returns:
            Gets notifications in Novu

        """
        payload = {
            "channels": channels or [],
            "templates": templates or [],
            "emails": emails or [],
            "subscriberIds": subscriber_ids or [],
            "search": search,
            "page": page,
            "transactionId": transaction_id,
        }
        return PaginatedActivityNotificationDto.from_camel_case(
            self.handle_request("GET", f"{self._notification_url}", payload=payload)
        )

    def stream(
        self,
        channels: Optional[List[str]] = None,
        templates: Optional[List[str]] = None,
        emails: Optional[List[str]] = None,
        subscriber_ids: Optional[List[str]] = None,
        search: Optional[str] = None,
        transaction_id: Optional[str] = None,
    ) -> PaginationIterator[ActivityNotificationDto]:
        """Stream all existing notifications into an iterator.

        Args:
            channels: A required parameter, should be an array of strings representing
                      available notification channels, such as "in_app", "email", "sms",
                      "chat", and "push".

            templates: A required parameter, should be an array of strings representing
                       the notification templates.

            emails: A required parameter, should be an array of strings representing
                    the email addresses associated with the notification.

            subscriber_ids: An array of subscriber's identifier associated with searched
                            notifications.

            search: A required parameter, should be a string representing the search query.

            transaction_id: A required parameter, should be a string representing the
                            transaction ID associated with the notification.

        Returns:
            An iterator on all notifications available.
        """
        payload = {
            "channels": channels or [],
            "templates": templates or [],
            "emails": emails or [],
            "subscriberIds": subscriber_ids or [],
            "search": search,
            "transactionId": transaction_id,
        }
        return PaginationIterator(self, ActivityNotificationDto, self._notification_url, payload=payload)

    def stats(self) -> Tuple[int, int]:
        """Gets notifications stats

        Returns:
            Tuple including weekly and monthly sent notifications.
        """
        data = self.handle_request("GET", f"{self._notification_url}/stats")["data"]
        return data["weeklySent"], data["monthlySent"]

    def graph_stats(
        self,
        id: Optional[str] = None,  # pylint: disable=C0103,W0622
        start_date: Optional[str] = None,
        end_date: Optional[str] = None,
        days: Optional[int] = None,
    ) -> Iterator[ActivityGraphStatesDto]:
        """Gets notifications graph stats.

        Args:
           id: is an optional parameter and should be a string. It represents the notification ID.
           start_date: is an optional parameter and should be a string. It represents the start date for the stats.
           end_date: is an optional parameter and should be a string. It represents the end date for the stats.
           days: is an optional parameter and should be an integer. It represents the number of days to get stats for.

        Returns:
           An iterator on notifications graph stats in Novu
        """
        payload = {
            "id": id,
            "start_date": start_date,
            "end_date": end_date,
            "days": days,
        }
        response = self.handle_request("GET", f"{self._notification_url}/graph/stats", payload=payload)
        for data in response["data"]:
            yield ActivityGraphStatesDto.from_camel_case(data)

    def get(self, notification_id: str) -> ActivityNotificationDto:
        """Trigger an event to get  notification by id"""
        url = f"{self._notification_url}/{notification_id}"
        response = self.handle_request("GET", url)
        return ActivityNotificationDto.from_camel_case(response["data"])



"""This module is used to gather enumerations related to the Template resource in Novu"""

from novu.enums.polyfill import StrEnum


class TemplateVariableTypeEnum(StrEnum):
    """This enumeration define possible type for a variable in a template"""

    STRING = "String"
    """String variable"""

    LIST = "Array"
    """List variable"""

    BOOL = "Boolean"
    """Boolean variable"""



"""
This module is used to define the ``EventApi``, a python wrapper to interact with ``Events`` in Novu.
"""

from collections.abc import Iterable
from typing import Dict, Iterable as _Iterable, List, Optional, Union

import requests

from novu.api.base import Api
from novu.constants import EVENTS_ENDPOINT
from novu.dto.event import EventDto, InputEventDto
from novu.dto.topic import TriggerTopicDto


class EventApi(Api):
    """This class aims to handle all API methods around events in Novu"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._event_url = f"{self._url}{EVENTS_ENDPOINT}"

    def trigger(
        self,
        name: str,
        recipients: Union[str, List[str]],
        payload: dict,
        overrides: Optional[dict] = None,
        transaction_id: Optional[str] = None,
        actor: Optional[str] = None,
        tenant: Optional[str] = None,
    ) -> EventDto:
        """Trigger event is the main way to send notification to subscribers.

        The trigger ID is used to match the particular template associated whit it.

        .. note:: If you want to use topic, please use
                  :meth:`~novu.api.event.EventApi.trigger_topic` instead

        .. warning:: For the "recipients" attribute, the part allowing the dynamic creation of
                     subscribers in the Novu AP was deliberately removed from the wrapper
                     to force you to pre-create them. It is for us a good practice that we must
                     begin to use for long-term programming with the platform (reuse of actor
                     during the sending, subscription, management of preferences ...).

        Args:
            name: The name of the template trigger to activate.
            recipients: A subscriber ID (or a list of subscriber ID) to reach with this trigger
            payload:
                A JSON serializable python dict to pass additional custom information that could be used to render the
                template, or perform routing rules based on it. This data will also be available when fetching the
                notifications feed from the API to display certain parts of the UI.

            overrides:
                A JSON serializable python dict used to override provider specific configurations. Defaults to None.

            transaction_id: A unique ID for this transaction, we will generated a UUID if not provided.

            actor:
                It is used to display the Avatar of the provided actor's subscriber id. Defaults to None.

            tenant:
                It is used to specify a tenant context during trigger event.
                If a new tenant object is provided, we will create a new tenant.

        Returns:
            Create Event definition in Novu
        """
        payload = {"name": name, "to": recipients, "payload": payload}
        if overrides:
            payload["overrides"] = overrides
        if actor:
            payload["actor"] = actor
        if transaction_id:
            payload["transactionId"] = transaction_id
        if tenant:
            payload["tenant"] = tenant

        return EventDto.from_camel_case(self.handle_request("POST", self._event_url, payload)["data"])

    def trigger_bulk(
        self,
        events: List[InputEventDto],
    ) -> List[EventDto]:
        """Trigger events in a bulk action to reduce the amount of api calls. Using this endpoint you can trigger
        multiple events at once, to avoid multiple calls to the API. The bulk API is limited to 100 events per request.

        Args:
            events (List[InputEventDto]): List of input events that should be sent.

        Returns:
            List[EventDto]: List of created Novu events.
        """
        payload: Dict[str, List[Dict[str, Union[str, List[str], dict]]]] = {"events": []}
        for event in events:
            event_payload = {"name": event.name, "to": event.recipients, "payload": event.payload}
            if event.overrides:
                event_payload["overrides"] = event.overrides
            if event.actor:
                event_payload["actor"] = event.actor
            if event.transaction_id:
                event_payload["transactionId"] = event.transaction_id
            if event.tenant:
                event_payload["tenant"] = event.tenant

            payload["events"].append(event_payload)

        # send trigger bulk events
        response = self.handle_request("POST", f"{self._event_url}/bulk", payload)["data"]

        triggered_events = []
        for triggered_event in response:
            triggered_events.append(EventDto.from_camel_case(triggered_event))

        return triggered_events

    def trigger_topic(
        self,
        name: str,
        topics: Union[TriggerTopicDto, _Iterable[TriggerTopicDto]],
        payload: dict,
        overrides: Optional[dict] = None,
        transaction_id: Optional[str] = None,
        actor: Optional[str] = None,
        tenant: Optional[str] = None,
    ) -> EventDto:
        """Trigger event is the main way to send notification to topic's subscribers.

        The trigger ID is used to match the particular template associated whit it.

        .. note:: If you want to use subscriber ID, please use :meth:`~novu.api.event.EventApi.trigger` instead

        Args:
            name: The name of the template trigger to activate.
            topics: A TriggerTopicDto (or a list of topic) to reach with this trigger
            payload:
                A JSON serializable python dict to pass additional custom information that could be used to render the
                template, or perform routing rules based on it. This data will also be available when fetching the
                notifications feed from the API to display certain parts of the UI.

            overrides:
                A JSON serializable python dict used to override provider specific configurations. Defaults to None.

            transaction_id: A unique ID for this transaction, we will generated a UUID if not provided.

            actor:
                It is used to display the Avatar of the provided actor's subscriber id. Defaults to None.

            tenant:
                It is used to specify a tenant context during trigger event.
                If a new tenant object is provided, we will create a new tenant.

        Returns:
            Create Event definition in Novu
        """
        _recipients = topics if isinstance(topics, Iterable) else [topics]

        payload = {"name": name, "to": [r.to_camel_case() for r in _recipients], "payload": payload}
        if overrides:
            payload["overrides"] = overrides
        if actor:
            payload["actor"] = actor
        if transaction_id:
            payload["transactionId"] = transaction_id
        if tenant:
            payload["tenant"] = tenant

        return EventDto.from_camel_case(self.handle_request("POST", self._event_url, payload)["data"])

    def broadcast(
        self,
        name: str,
        payload: dict,
        overrides: Optional[dict] = None,
        transaction_id: Optional[str] = None,
        actor: Optional[str] = None,
        tenant: Optional[str] = None,
    ):
        """Trigger a broadcast event to all existing subscribers, could be used to send announcements, etc.

        Args:
            name: The name of the template trigger to activate.
            payload:
                A JSON serializable python dict to pass additional custom information that could be used to render the
                template, or perform routing rules based on it. This data will also be available when fetching the
                notifications feed from the API to display certain parts of the UI.

            overrides:
                A JSON serializable python dict used to override provider specific configurations. Defaults to None.

            transaction_id: A unique ID for this transaction, we will generated a UUID if not provided.

            actor:
                It is used to display the Avatar of the provided actor's subscriber id. Defaults to None.

            tenant:
                It is used to specify a tenant context during trigger event.
                If a new tenant object is provided, we will create a new tenant.

        Returns:
            Create Event definition in Novu
        """
        payload = {"name": name, "payload": payload}
        if overrides:
            payload["overrides"] = overrides
        if actor:
            payload["actor"] = actor
        if transaction_id:
            payload["transactionId"] = transaction_id
        if tenant:
            payload["tenant"] = tenant

        return EventDto.from_camel_case(self.handle_request("POST", f"{self._event_url}/broadcast", payload)["data"])

    def delete(self, transaction_id: str) -> None:
        """Using a previously generated transaction ID during the event trigger, will cancel any active or pending
        workflows. This is useful to cancel active digests, delays, etc...

        Args:
            transaction_id: The transaction ID to cancel workflows
        """
        self.handle_request("DELETE", f"{self._event_url}/{transaction_id}")



"""This module is used to gather all constants and magic numbers used in the Novu client."""

# API Endpoints
CHANGES_ENDPOINT = "/v1/changes"
ENVIRONMENTS_ENDPOINT = "/v1/environments"
EVENTS_ENDPOINT = "/v1/events/trigger"
EXECUTION_DETAILS_ENDPOINT = "/v1/execution-details"
FEEDS_ENDPOINT = "/v1/feeds"
INBOUND_PARSE_ENDPOINT = "/v1/inbound-parse"
INTEGRATIONS_ENDPOINT = "/v1/integrations"
LAYOUTS_ENDPOINT = "/v1/layouts"
MESSAGES_ENDPOINT = "/v1/messages"
NOTIFICATION_ENDPOINT = "/v1/notifications"
NOTIFICATION_GROUPS_ENDPOINT = "/v1/notification-groups"
NOTIFICATION_TEMPLATES_ENDPOINT = "/v1/notification-templates"
ORGANIZATION_ENDPOINT = "/v1/organizations"
SUBSCRIBERS_ENDPOINT = "/v1/subscribers"
TENANTS_ENDPOINT = "/v1/tenants"
TOPICS_ENDPOINT = "/v1/topics"
BLUEPRINTS_ENDPOINT = "/v1/blueprints"
WORKFLOW_ENDPOINT = "/v1/workflows"

# Datetime Format
DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%f%z"



"""This module is used to gather all DTO definitions related to the Tenant resource in Novu"""

import dataclasses
from typing import Optional

from novu.dto.base import CamelCaseDto, DtoIterableDescriptor


@dataclasses.dataclass
class TenantDto(CamelCaseDto["TenantDto"]):
    """Tenant definition"""

    identifier: str
    """Tenant identifier"""

    name: str
    """Tenant name"""

    data: Optional[dict] = None
    """Unstructured dictionary of data stored on the tenant"""

    _id: Optional[str] = None
    """Topic ID in Novu internal system"""

    _environment_id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    created_at: Optional[str] = None
    """Creation date of the subscriber"""

    updated_at: Optional[str] = None
    """Last update date of the subscriber"""


@dataclasses.dataclass
class PaginatedTenantDto(CamelCaseDto["PaginatedTenantDto"]):
    """Paginated topic definition"""

    page: int = 0
    """Page number"""

    has_more: bool = False
    """Does the list have more items to fetch"""

    page_size: int = 0
    """Page size"""

    data: DtoIterableDescriptor[TenantDto] = DtoIterableDescriptor[TenantDto](default_factory=list, item_cls=TenantDto)
    """Data"""



"""This module is used to gather all DTO definitions related to the Step Filter resource in Novu"""

import dataclasses
from typing import Optional

from novu.dto.base import CamelCaseDto, DtoIterableDescriptor
from novu.dto.field import FieldFilterPartDto
from novu.enums.step_filter import StepFilterType, StepFilterValue


@dataclasses.dataclass
class StepFilterDto(CamelCaseDto["StepFilterDto"]):
    """Definition of a step filter"""

    is_negated: Optional[bool] = None
    """If the filter should be negated"""

    type: Optional[StepFilterType] = None
    """Step filter's type"""

    value: Optional[StepFilterValue] = None
    """Step filter's value"""

    children: DtoIterableDescriptor[FieldFilterPartDto] = DtoIterableDescriptor[FieldFilterPartDto](
        default_factory=list, item_cls=FieldFilterPartDto
    )
    """Step filter's children"""



"""This module is used to gather enumerations related to the Provider resource in Novu"""

from novu.enums.polyfill import StrEnum


class MemberRole(StrEnum):
    """This enumeration define possible role for a member"""

    ADMIN = "admin"
    """The member is an administrator"""

    MEMBER = "member"
    """The member is a simple member"""


class MemberStatus(StrEnum):
    """This enumeration define possible status for a member"""

    NEW = "new"
    """The member is new"""

    ACTIVE = "active"
    """The member is active"""

    INVITED = "invited"
    """The member has been invited"""



"""Polyfill of backports related to enums."""

try:
    from enum import StrEnum  # type: ignore[attr-defined]
except ImportError:
    from enum import Enum

    class StrEnum(str, Enum):  # type: ignore[no-redef]
        """Redefinition of ``StrEnum`` to backport this class from python 3.10."""


__all__ = ["StrEnum"]



"""This module is used to gather all DTO definitions related to the Environment resource in Novu"""

import dataclasses
from typing import Optional

from novu.dto.base import CamelCaseDto, DtoDescriptor, DtoIterableDescriptor


@dataclasses.dataclass
class EnvironmentWidgetDto(CamelCaseDto["EnvironmentWidgetDto"]):
    """Definition of an environment's widget"""

    notification_center_encryption: bool
    """If the notification center use encryption"""


@dataclasses.dataclass
class EnvironmentApiKeyDto(CamelCaseDto["EnvironmentApiKeyDto"]):
    """Definition of an environment's api-key"""

    key: str
    """The API key"""

    _user_id: str
    """The user identifier related to the api-key"""

    _id: Optional[str] = None
    """Environment's API Key ID in Novu internal storage system"""


@dataclasses.dataclass
class EnvironmentDto(CamelCaseDto["EnvironmentDto"]):  # pylint: disable=R0902
    """Definition of an environment"""

    id: Optional[str] = None  # pylint: disable=C0103
    """Environment ID in Novu internal storage system"""

    _id: Optional[str] = None
    """Environment ID in Novu internal storage system"""

    name: Optional[str] = None
    """Name of the environment"""

    identifier: Optional[str] = None
    """Identifier of the environment"""

    _organization_id: Optional[str] = None
    """Organization ID in Novu internal storage system"""

    widget: DtoDescriptor[EnvironmentWidgetDto] = DtoDescriptor[EnvironmentWidgetDto](item_cls=EnvironmentWidgetDto)
    """Environment's widget settings"""

    api_keys: DtoIterableDescriptor[EnvironmentApiKeyDto] = DtoIterableDescriptor[EnvironmentApiKeyDto](
        default_factory=list, item_cls=EnvironmentApiKeyDto
    )
    """List of API Keys related to the environment"""

    created_at: Optional[str] = None
    """Creation date of the environment"""

    updated_at: Optional[str] = None
    """Last update date of the environment"""



"""
This module is used to define the ``NotificationGroupApi``, a python wrapper
to interact with ``NotificationGroup`` in Novu.
"""

from typing import Optional

import requests

from novu.api.base import Api
from novu.constants import NOTIFICATION_GROUPS_ENDPOINT
from novu.dto.notification_group import (
    NotificationGroupDto,
    PaginatedNotificationGroupDto,
)


class NotificationGroupApi(Api):
    """This class aims to handle all API methods around notification groups in API"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._notification_group_url = f"{self._url}{NOTIFICATION_GROUPS_ENDPOINT}"

    def list(self) -> PaginatedNotificationGroupDto:
        """Method to list notification groups

        Returns:
            Paginated notification groups
        """
        return PaginatedNotificationGroupDto.from_camel_case(self.handle_request("GET", self._notification_group_url))

    def create(self, name: str) -> NotificationGroupDto:
        """Method to create a new notification group

        Args:
            name: The name of the notification group.

        Returns:
            The created notification group.
        """
        return NotificationGroupDto.from_camel_case(
            self.handle_request("POST", self._notification_group_url, {"name": name})["data"]
        )

    def get(self, _id: str) -> NotificationGroupDto:
        """Method to get a notification group

        Args:
            _id: The id of the notification group.

        Returns:
            The notification group.
        """
        return NotificationGroupDto.from_camel_case(
            self.handle_request("GET", f"{self._notification_group_url}/{_id}")["data"]
        )

    def patch(self, _id: str, name: str) -> NotificationGroupDto:
        """Method to patch a notification group

        Args:
            _id: The id of the notification group.
            name: The name of the notification group.

        Returns:
            The patched notification group.
        """
        return NotificationGroupDto.from_camel_case(
            self.handle_request("PATCH", f"{self._notification_group_url}/{_id}", {"name": name})["data"]
        )

    def delete(self, _id: str) -> NotificationGroupDto:
        """Method to delete a notification group

        Args:
            _id: The id of the notification group.

        Returns:
            The deleted notification group.
        """
        return self.handle_request("DELETE", f"{self._notification_group_url}/{_id}")["data"]



"""This module is used to gather all DTO definitions related to the Organization resource in Novu.

An organization serves as a separate entity within your Novu account.
Each organization you create has its own separate integration store, workflows, subscribers, and API keys.
This separation of resources allows you to manage multi-tenant environments and
separate domains within a single account.
"""

import dataclasses
from typing import List, Optional

from novu.dto.base import CamelCaseDto, DtoDescriptor, DtoIterableDescriptor
from novu.enums import OrganizationBrandingDirection, PartnerTypeEnum


@dataclasses.dataclass
class PartnerConfigurationDto(CamelCaseDto["PartnerConfigurationDto"]):
    """Organization partner configuration definition"""

    access_token: str
    """Access token"""

    configuration_id: str
    """Configuration identifier"""

    partner_type: PartnerTypeEnum
    """Partner type"""

    team_id: Optional[str] = None
    """Partner team identifier"""

    project_ids: Optional[List[str]] = None
    """Partner's project identifiers"""


@dataclasses.dataclass
class OrganizationBrandingDto(CamelCaseDto["OrganizationBrandingDto"]):
    """Organization branding definition"""

    logo: Optional[str] = None
    """Branding logo"""

    color: Optional[str] = None
    """Branding color"""

    font_color: Optional[str] = None
    """Branding font color"""

    content_background: Optional[str] = None
    """Branding background content"""

    direction: Optional[OrganizationBrandingDirection] = None
    """Branding direction"""

    font_family: Optional[str] = None
    """Branding font-family"""


@dataclasses.dataclass
class OrganizationDto(CamelCaseDto["OrganizationDto"]):  # pylint: disable=R0902
    """Organization definition"""

    name: str
    """Organization name"""

    branding: DtoDescriptor[OrganizationBrandingDto] = DtoDescriptor[OrganizationBrandingDto](
        item_cls=OrganizationBrandingDto
    )
    """Organization branding"""

    partner_configurations: DtoIterableDescriptor[PartnerConfigurationDto] = DtoIterableDescriptor[
        PartnerConfigurationDto
    ](default_factory=list, item_cls=PartnerConfigurationDto)
    """List of partner configurations used in the organization"""

    logo: Optional[str] = None
    """Organization's logo (S3 signed URL if you read information from API)"""



"""
This module is used to define the ``TenantApi``, a python wrapper to interact with ``Tenants`` in Novu.
"""

from typing import Dict, Optional, Union

import requests

from novu.api.base import Api, PaginationIterator
from novu.constants import TENANTS_ENDPOINT
from novu.dto.tenant import PaginatedTenantDto, TenantDto


class TenantApi(Api):
    """This class aims to handle all API methods around tenants in API"""

    def __init__(
        self,
        url: Optional[str] = None,
        api_key: Optional[str] = None,
        requests_timeout: Optional[int] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        super().__init__(url=url, api_key=api_key, requests_timeout=requests_timeout, session=session)

        self._tenant_url = f"{self._url}{TENANTS_ENDPOINT}"

    def list(self, page: Optional[int] = None, limit: Optional[int] = None) -> PaginatedTenantDto:
        """List existing tenants

        Args:
            page: Page to retrieve. Defaults to None.
            limit: Size of the page to retrieve. Defaults to None.

        Returns:
            Paginated list of tenant
        """
        payload: Dict[str, Union[int, str]] = {}
        if page:
            payload["page"] = page
        if limit:
            payload["limit"] = limit

        return PaginatedTenantDto.from_camel_case(self.handle_request("GET", self._tenant_url, payload=payload))

    def stream(self) -> PaginationIterator[TenantDto]:
        """Stream all existing tenants into an iterator.

        Returns:
            An iterator on all tenants available.
        """
        return PaginationIterator(self, TenantDto, self._tenant_url)

    def create(self, identifier: str, name: str, data: Optional[dict] = None) -> TenantDto:
        """Create a tenant

        Args:
            identifier: Identifier of the new tenant
            name: Name of the new tenant
            data: Data of the new tenant. Default to None.

        Returns:
            Created tenant
        """
        return TenantDto.from_camel_case(
            self.handle_request("POST", self._tenant_url, {"identifier": identifier, "name": name, "data": data or {}})[
                "data"
            ]
        )

    def get(self, identifier: str) -> TenantDto:
        """Retrieve a Tenant using his identifier

        .. note:: This method is the easiest way to get the list of subscribers.

        Args:
            identifier: The tenant identifier (ID in Novu)

        Returns:
            TenantDto
        """
        return TenantDto.from_camel_case(self.handle_request("GET", f"{self._tenant_url}/{identifier}")["data"])

    def patch(
        self, reference: str, identifier: Optional[str] = None, name: Optional[str] = None, data: Optional[dict] = None
    ) -> TenantDto:
        """Edit a Tenant using his identifier

        Args:
            reference: Current identifier of the tenant in Novu
            identifier: New identifier to set on the tenant
            name: New name to set on the tenant
            data: New data dict to set on the tenant

        Returns:
            TenantDto
        """
        payload = {"identifier": identifier, "name": name, "data": data}
        payload = {k: v for k, v in payload.items() if v}
        return TenantDto.from_camel_case(
            self.handle_request("PATCH", f"{self._tenant_url}/{reference}", payload)["data"]
        )

    def delete(self, identifier: str) -> None:
        """Remove a Tenant using his identifier

        Args:
            identifier: Current identifier of the tenant in Novu
        """
        self.handle_request("DELETE", f"{self._tenant_url}/{identifier}")
